"""Tests for shell detection and integration module."""

from __future__ import annotations

import os
from unittest.mock import patch

from dekk.shell import (
    ActivationConfig,
    ActivationScriptBuilder,
    AliasSuggestor,
    CommandArg,
    CommandFlag,
    CompletionGenerator,
    CompletionSpec,
    EnvVar,
    PromptHelper,
    ShellDetector,
    ShellInfo,
    ShellKind,
    Subcommand,
)

# ---------------------------------------------------------------------------
# ShellDetector
# ---------------------------------------------------------------------------


class TestShellDetector:
    def setup_method(self):
        self.detector = ShellDetector()

    def test_detect_with_bash_override(self):
        info = self.detector.detect(shell_override="bash")
        assert info.kind == ShellKind.BASH

    def test_detect_with_zsh_override(self):
        info = self.detector.detect(shell_override="zsh")
        assert info.kind == ShellKind.ZSH

    def test_detect_with_fish_override(self):
        info = self.detector.detect(shell_override="fish")
        assert info.kind == ShellKind.FISH

    def test_detect_with_tcsh_override(self):
        info = self.detector.detect(shell_override="tcsh")
        assert info.kind == ShellKind.TCSH

    def test_detect_with_path_override(self):
        info = self.detector.detect(shell_override="/bin/bash")
        assert info.kind == ShellKind.BASH

    def test_detect_with_powershell_override(self):
        info = self.detector.detect(shell_override="pwsh")
        assert info.kind == ShellKind.PWSH

    @patch.dict(os.environ, {"SHELL": "/bin/bash"})
    def test_detect_from_env(self):
        info = self.detector.detect()
        assert info.kind == ShellKind.BASH
        assert info.login_shell == "/bin/bash"

    @patch.dict(os.environ, {"SHELL": "/usr/bin/zsh"})
    def test_detect_zsh_from_env(self):
        info = self.detector.detect()
        assert info.kind == ShellKind.ZSH

    @patch.dict(os.environ, {"SHELL": "/usr/bin/fish"})
    def test_detect_fish_from_env(self):
        info = self.detector.detect()
        assert info.kind == ShellKind.FISH

    @patch.dict(os.environ, {"SHELL": "/tool/pandora/bin/tcsh"})
    def test_detect_tcsh_from_env(self):
        info = self.detector.detect()
        assert info.kind == ShellKind.TCSH

    def test_detect_never_raises(self):
        with patch.dict(os.environ, {}, clear=True):
            # Even with no SHELL env var, should not raise
            info = self.detector.detect()
            assert isinstance(info, ShellInfo)

    def test_shell_info_is_posix(self):
        info = ShellInfo(kind=ShellKind.BASH)
        assert info.is_posix
        assert not info.is_fish
        assert not info.is_powershell
        assert not info.is_csh_family

    def test_shell_info_is_fish(self):
        info = ShellInfo(kind=ShellKind.FISH)
        assert info.is_fish
        assert not info.is_posix

    def test_shell_info_is_powershell(self):
        info = ShellInfo(kind=ShellKind.POWERSHELL)
        assert info.is_powershell
        assert not info.is_posix

    def test_shell_info_is_tcsh(self):
        info = ShellInfo(kind=ShellKind.TCSH)
        assert info.is_csh_family
        assert not info.is_posix

    def test_supports_functions(self):
        assert ShellInfo(kind=ShellKind.BASH).supports_functions
        assert ShellInfo(kind=ShellKind.ZSH).supports_functions
        assert ShellInfo(kind=ShellKind.FISH).supports_functions
        assert not ShellInfo(kind=ShellKind.SH).supports_functions
        assert not ShellInfo(kind=ShellKind.DASH).supports_functions


# ---------------------------------------------------------------------------
# ActivationScriptBuilder
# ---------------------------------------------------------------------------


class TestActivationScriptBuilder:
    def setup_method(self):
        self.builder = ActivationScriptBuilder()
        self.config = ActivationConfig(
            env_vars=(
                EnvVar(name="MLIR_DIR", value="/opt/conda/lib/cmake/mlir"),
                EnvVar(name="LLVM_DIR", value="/opt/conda/lib/cmake/llvm"),
            ),
            path_prepends=("/opt/conda/bin",),
            app_name="apxm",
        )

    def test_posix_activation(self):
        script = self.builder.build(self.config, ShellKind.BASH)
        assert 'export MLIR_DIR="/opt/conda/lib/cmake/mlir"' in script
        assert 'export LLVM_DIR="/opt/conda/lib/cmake/llvm"' in script
        assert 'export PATH="/opt/conda/bin:$PATH"' in script
        assert "# Activation script for apxm" in script

    def test_posix_saves_old_values(self):
        script = self.builder.build(self.config, ShellKind.BASH)
        assert "_OLD_MLIR_DIR=" in script
        assert "_OLD_LLVM_DIR=" in script
        assert "_OLD_PATH=" in script

    def test_posix_deactivation(self):
        script = self.builder.build_deactivate(self.config, ShellKind.BASH)
        assert "# Deactivation script for apxm" in script
        assert "unset _OLD_MLIR_DIR" in script
        assert "unset MLIR_DIR" in script

    def test_zsh_same_as_posix(self):
        bash_script = self.builder.build(self.config, ShellKind.BASH)
        zsh_script = self.builder.build(self.config, ShellKind.ZSH)
        assert bash_script == zsh_script

    def test_fish_activation(self):
        script = self.builder.build(self.config, ShellKind.FISH)
        assert "set -gx MLIR_DIR /opt/conda/lib/cmake/mlir" in script
        assert "set -gx LLVM_DIR /opt/conda/lib/cmake/llvm" in script
        assert "set -gx PATH /opt/conda/bin $PATH" in script

    def test_fish_deactivation(self):
        script = self.builder.build_deactivate(self.config, ShellKind.FISH)
        assert "set -e _OLD_MLIR_DIR" in script

    def test_tcsh_activation(self):
        script = self.builder.build(self.config, ShellKind.TCSH)
        assert 'setenv MLIR_DIR "/opt/conda/lib/cmake/mlir"' in script
        assert 'setenv LLVM_DIR "/opt/conda/lib/cmake/llvm"' in script
        assert 'setenv PATH "/opt/conda/bin:$PATH"' in script

    def test_tcsh_deactivation(self):
        script = self.builder.build_deactivate(self.config, ShellKind.TCSH)
        assert "unsetenv MLIR_DIR" in script

    def test_powershell_activation(self):
        script = self.builder.build(self.config, ShellKind.POWERSHELL)
        assert '$env:MLIR_DIR = "/opt/conda/lib/cmake/mlir"' in script
        assert '$env:LLVM_DIR = "/opt/conda/lib/cmake/llvm"' in script

    def test_powershell_deactivation(self):
        script = self.builder.build_deactivate(self.config, ShellKind.POWERSHELL)
        assert "Remove-Item Env:MLIR_DIR" in script

    def test_prepend_path_var(self):
        config = ActivationConfig(
            env_vars=(EnvVar(name="LD_LIBRARY_PATH", value="/opt/lib", prepend_path=True),),
            app_name="test",
        )
        script = self.builder.build(config, ShellKind.BASH)
        assert 'export LD_LIBRARY_PATH="/opt/lib${LD_LIBRARY_PATH:+:$LD_LIBRARY_PATH}"' in script

    def test_banner(self):
        config = ActivationConfig(
            env_vars=(),
            app_name="test",
            banner="APXM environment activated",
        )
        script = self.builder.build(config, ShellKind.BASH)
        assert 'echo "APXM environment activated"' in script

    def test_fish_banner(self):
        config = ActivationConfig(
            env_vars=(),
            app_name="test",
            banner="APXM environment activated",
        )
        script = self.builder.build(config, ShellKind.FISH)
        assert 'echo "APXM environment activated"' in script

    def test_empty_config(self):
        config = ActivationConfig()
        script = self.builder.build(config, ShellKind.BASH)
        assert "Activation script" in script

    def test_apxm_realistic_config(self):
        """Test with the actual APXM activation config pattern."""
        prefix = "/home/user/miniforge3/envs/apxm"
        config = ActivationConfig(
            env_vars=(
                EnvVar(name="MLIR_DIR", value=f"{prefix}/lib/cmake/mlir"),
                EnvVar(name="LLVM_DIR", value=f"{prefix}/lib/cmake/llvm"),
                EnvVar(name="MLIR_PREFIX", value=prefix),
                EnvVar(name="LLVM_PREFIX", value=prefix),
            ),
            path_prepends=(f"{prefix}/bin",),
            app_name="apxm",
            banner="APXM MLIR/LLVM toolchain activated",
        )

        # Test all shells
        for shell in (
            ShellKind.BASH,
            ShellKind.ZSH,
            ShellKind.FISH,
            ShellKind.TCSH,
            ShellKind.POWERSHELL,
        ):
            script = self.builder.build(config, shell)
            assert prefix in script
            assert "mlir" in script.lower() or "MLIR" in script


# ---------------------------------------------------------------------------
# CompletionGenerator
# ---------------------------------------------------------------------------


class TestCompletionGenerator:
    def setup_method(self):
        self.generator = CompletionGenerator()
        self.spec = CompletionSpec(
            command="apxm",
            description="APxM CLI",
            flags=(
                CommandFlag(long="--config", description="Config file path", takes_value=True),
                CommandFlag(
                    long="--trace",
                    description="Enable tracing",
                    takes_value=True,
                    choices=("trace", "debug", "info", "warn", "error"),
                ),
            ),
            subcommands=(
                Subcommand(
                    name="compile",
                    description="Compile ApxmGraph to artifact",
                    flags=(
                        CommandFlag(
                            long="--output", short="-o", description="Output path", takes_value=True
                        ),
                        CommandFlag(
                            long="--opt-level",
                            short="-O",
                            description="Optimization level",
                            takes_value=True,
                            choices=("0", "1", "2", "3"),
                        ),
                        CommandFlag(
                            long="--emit-diagnostics",
                            description="Emit diagnostics JSON",
                            takes_value=True,
                        ),
                    ),
                    args=(
                        CommandArg(
                            name="input", description="Input graph file", file_completion=True
                        ),
                    ),
                ),
                Subcommand(
                    name="execute",
                    description="Compile and run a graph",
                    flags=(
                        CommandFlag(
                            long="--opt-level",
                            short="-O",
                            description="Optimization level",
                            takes_value=True,
                            choices=("0", "1", "2", "3"),
                        ),
                        CommandFlag(
                            long="--emit-metrics", description="Emit metrics JSON", takes_value=True
                        ),
                    ),
                    args=(
                        CommandArg(
                            name="input", description="Input graph file", file_completion=True
                        ),
                    ),
                ),
                Subcommand(name="doctor", description="Diagnose dependencies"),
                Subcommand(
                    name="activate",
                    description="Print shell env exports",
                    flags=(
                        CommandFlag(
                            long="--shell",
                            description="Shell format",
                            takes_value=True,
                            choices=("sh", "bash", "zsh", "fish", "tcsh"),
                        ),
                    ),
                ),
                Subcommand(name="install", description="Install conda environment"),
                Subcommand(
                    name="register",
                    description="Manage LLM credentials",
                    subcommands=(
                        Subcommand(name="add", description="Add a credential"),
                        Subcommand(name="list", description="List credentials"),
                        Subcommand(name="remove", description="Remove a credential"),
                        Subcommand(name="test", description="Test credentials"),
                    ),
                ),
            ),
        )

    def test_bash_completion(self):
        script = self.generator.generate(self.spec, ShellKind.BASH)
        assert "__apxm_completions" in script
        assert "complete -F" in script
        assert "compile" in script
        assert "execute" in script
        assert "doctor" in script
        assert "activate" in script

    def test_zsh_completion(self):
        script = self.generator.generate(self.spec, ShellKind.ZSH)
        assert "#compdef apxm" in script
        assert "_apxm" in script
        assert "compile" in script
        assert "execute" in script

    def test_fish_completion(self):
        script = self.generator.generate(self.spec, ShellKind.FISH)
        assert "complete -c apxm" in script
        assert "compile" in script
        assert "opt-level" in script  # fish uses -l opt-level (no -- prefix)

    def test_powershell_completion(self):
        script = self.generator.generate(self.spec, ShellKind.POWERSHELL)
        assert "Register-ArgumentCompleter" in script
        assert "apxm" in script
        assert "compile" in script

    def test_unsupported_shell(self):
        script = self.generator.generate(self.spec, ShellKind.TCSH)
        assert "not supported" in script

    def test_nested_subcommands_in_bash(self):
        script = self.generator.generate(self.spec, ShellKind.BASH)
        assert "register" in script
        assert "add" in script  # nested under register


# ---------------------------------------------------------------------------
# PromptHelper
# ---------------------------------------------------------------------------


class TestPromptHelper:
    def setup_method(self):
        self.helper = PromptHelper()

    def test_bash_prompt(self):
        snippet = self.helper.status_snippet(ShellKind.BASH)
        assert "SNIFF_STATUS" in snippet

    def test_fish_prompt(self):
        snippet = self.helper.status_snippet(ShellKind.FISH)
        assert "set -q SNIFF_STATUS" in snippet

    def test_powershell_prompt(self):
        snippet = self.helper.status_snippet(ShellKind.POWERSHELL)
        assert "$env:SNIFF_STATUS" in snippet

    def test_custom_env_var(self):
        snippet = self.helper.status_snippet(ShellKind.BASH, env_var="APXM_ENV")
        assert "APXM_ENV" in snippet


# ---------------------------------------------------------------------------
# AliasSuggestor
# ---------------------------------------------------------------------------


class TestAliasSuggestor:
    def setup_method(self):
        self.suggestor = AliasSuggestor()

    def test_suggest_basic(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile", "execute", "doctor"])
        assert len(suggestions) > 0
        aliases = [s.alias for s in suggestions]
        # Should have a short alias for the base command
        assert "ap" in aliases

    def test_suggest_subcommand_aliases(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile", "execute"])
        aliases = [s.alias for s in suggestions]
        assert "ac" in aliases  # apxm compile -> ac
        assert "ae" in aliases  # apxm execute -> ae

    def test_render_posix(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile"])
        rendered = self.suggestor.render(suggestions, ShellKind.BASH)
        assert "alias " in rendered

    def test_render_fish(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile"])
        rendered = self.suggestor.render(suggestions, ShellKind.FISH)
        assert "alias " in rendered

    def test_render_tcsh(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile"])
        rendered = self.suggestor.render(suggestions, ShellKind.TCSH)
        assert "alias " in rendered

    def test_render_powershell(self):
        suggestions = self.suggestor.suggest("apxm", subcommands=["compile"])
        rendered = self.suggestor.render(suggestions, ShellKind.POWERSHELL)
        assert "Set-Alias" in rendered

    def test_common_flags(self):
        suggestions = self.suggestor.suggest(
            "apxm",
            common_flags={"v": "--trace debug"},
        )
        found = [s for s in suggestions if s.alias == "apxmv"]
        assert len(found) == 1
        assert "--trace debug" in found[0].command




"""Conda-specific toolchain profile."""

from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path

from dekk.execution.os import get_dekk_os
from dekk.execution.toolchain.builder import EnvVarBuilder

CONDA_PREFIX_ENV = "CONDA_PREFIX"
CONDA_DEFAULT_ENV = "CONDA_DEFAULT_ENV"


@dataclass(frozen=True)
class CondaToolchain:
    """Conda environment toolchain."""

    prefix: Path
    env_name: str = ""

    @property
    def path_dirs(self) -> tuple[Path, ...]:
        return get_dekk_os().conda_runtime_paths(self.prefix)

    def configure(self, builder: EnvVarBuilder) -> None:
        builder.set_var(CONDA_PREFIX_ENV, str(self.prefix))
        if self.env_name:
            builder.set_var(CONDA_DEFAULT_ENV, self.env_name)
        for path in self.path_dirs:
            builder.prepend_path(path)


__all__ = ["CondaToolchain"]



"""Configuration management with TOML support and multi-tier precedence.

Provides a :class:`ConfigManager` that loads configuration from multiple
tiers (user config, project config, environment variables) and supports
reading/writing TOML files with dot-notation access to nested keys.

Tier precedence (highest to lowest):
    1. Environment variables  (``{APP_NAME}_SECTION_KEY``)
    2. Project config         (``.{app_name}/config.toml`` in ancestor dirs)
    3. User config            (platform-standard user config dir)
    4. Built-in defaults      (passed to constructor)
"""

from __future__ import annotations

import os
from collections.abc import Mapping
from pathlib import Path
from typing import Any, Final

from dekk._compat import deep_merge, load_toml, tomllib
from dekk.core.paths import find_project_config_file, project_config_file, user_config_file

# TOML writing: prefer tomli_w when available, fall back to a small writer.
try:
    import tomli_w
except ImportError:
    tomli_w = None  # type: ignore[assignment]


DEFAULT_CONFIG_FILE: Final = "config.toml"
KEY_SEPARATOR: Final = "."
ENV_VAR_SEPARATOR: Final = "_"


class ConfigManager:
    """Multi-tier configuration manager with TOML support.

    Loads configuration from user config, project config, and environment
    variables, with higher tiers overriding lower ones.  Supports dot-notation
    for nested key access (e.g. ``"database.path"``).

    Args:
        app_name: Application name used for directory/env-var naming.
        config_file: Config filename within the app config directory.
        defaults: Built-in default values to start with.

    Example::

        cfg = ConfigManager("myapp")
        cfg.set("database.path", "/tmp/db.sqlite")
        cfg.save()
        print(cfg.get("database.path"))  # /tmp/db.sqlite
    """

    def __init__(
        self,
        app_name: str,
        config_file: str = DEFAULT_CONFIG_FILE,
        defaults: dict[str, Any] | None = None,
    ) -> None:
        self.app_name = app_name
        self.config_file = config_file
        self._defaults: dict[str, Any] = defaults or {}
        self._config: dict[str, Any] = {}
        self.load()

    # ------------------------------------------------------------------
    # Public API
    # ------------------------------------------------------------------

    def load(self) -> None:
        """Load config from all tiers (defaults -> user -> project -> env vars).

        Each tier deeply merges into the accumulated config so that
        higher-precedence tiers override individual keys without wiping
        entire sections.
        """
        # Start from built-in defaults
        config: dict[str, Any] = _deep_copy(self._defaults)

        # User config: platform-standard user config directory
        user_config = self._user_config_path()
        if user_config.exists():
            config = deep_merge(config, load_toml(user_config) or {})

        # Project config: .{app_name}/config.toml (search ancestors)
        project_config = self._find_project_config()
        if project_config is not None:
            config = deep_merge(config, load_toml(project_config) or {})

        # Environment variables: {APP_NAME}_{KEY}
        config = deep_merge(config, self._load_env_vars())

        self._config = config

    def get(self, key: str, default: Any = None) -> Any:
        """Get a config value using dot notation.

        Args:
            key: Dot-separated path (e.g. ``"database.path"``).
            default: Returned when the key is missing.

        Returns:
            The stored value, or *default* if not found.
        """
        parts = key.split(KEY_SEPARATOR)
        value: Any = self._config
        for part in parts:
            if isinstance(value, dict) and part in value:
                value = value[part]
            else:
                return default
        return value

    def set(self, key: str, value: Any) -> None:
        """Set a config value using dot notation.

        Intermediate dicts are created automatically.

        Args:
            key: Dot-separated path (e.g. ``"database.path"``).
            value: Value to store.
        """
        parts = key.split(KEY_SEPARATOR)
        target = self._config
        for part in parts[:-1]:
            if part not in target or not isinstance(target[part], dict):
                target[part] = {}
            target = target[part]
        target[parts[-1]] = value

    def save(self, user: bool = True) -> None:
        """Save the current config to a TOML file.

        Args:
            user: If ``True`` (default), write to the user config directory
                for the current platform. If ``False``, write to the
                project-local directory (``.{app_name}/config.toml`` relative
                to the current working directory).

        """
        if user:
            path = self._user_config_path()
        else:
            path = project_config_file(self.app_name, config_file=self.config_file)

        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_text(_dump_toml(self._config), encoding="utf-8")

    def to_dict(self) -> dict[str, Any]:
        """Return the full configuration as a dictionary (shallow copy)."""
        return self._config.copy()

    # ------------------------------------------------------------------
    # Helpers
    # ------------------------------------------------------------------

    def _user_config_path(self) -> Path:
        """Return the user-level config file path."""
        return user_config_file(self.app_name, self.config_file)

    def _find_project_config(self) -> Path | None:
        """Search ancestor directories for a project config file.

        Walks up from the current working directory looking for
        ``.{app_name}/config.toml``.

        Returns:
            Path to the project config, or ``None`` if not found.
        """
        return find_project_config_file(self.app_name, config_file=self.config_file)

    def _load_env_vars(self) -> dict[str, Any]:
        """Load config overrides from environment variables.

        Variables matching ``{APP_NAME}_*`` are converted to nested keys:
        ``MYAPP_DATABASE_PATH`` becomes ``{"database": {"path": "..."}}``.

        Returns:
            Dict of config values derived from the environment.
        """
        prefix = f"{self.app_name.upper()}{ENV_VAR_SEPARATOR}"
        config: dict[str, Any] = {}
        for key, value in os.environ.items():
            if key.startswith(prefix):
                config_key = key[len(prefix) :].lower()
                # Build nested dict from underscore-separated parts
                parts = config_key.split(ENV_VAR_SEPARATOR)
                target = config
                for part in parts[:-1]:
                    if part not in target or not isinstance(target[part], dict):
                        target[part] = {}
                    target = target[part]
                target[parts[-1]] = value
        return config


# ------------------------------------------------------------------
# Module-level helpers
# ------------------------------------------------------------------


def _deep_copy(d: dict[str, Any]) -> dict[str, Any]:
    """Simple deep copy for nested dicts of plain values."""
    result: dict[str, Any] = {}
    for key, value in d.items():
        if isinstance(value, dict):
            result[key] = _deep_copy(value)
        elif isinstance(value, list):
            result[key] = list(value)
        else:
            result[key] = value
    return result


def _load_toml(path: Path) -> dict[str, Any]:
    """Load a TOML file, returning an empty dict on error.

    Thin wrapper kept for backward compatibility; delegates to
    :func:`dekk._compat.load_toml`.
    """
    if tomllib is None:
        return {}
    return load_toml(path) or {}


def _dump_toml(data: dict[str, Any]) -> str:
    """Serialize a nested config dict to TOML."""
    if tomli_w is not None:
        return tomli_w.dumps(data)
    return _render_toml_table(data)


def _render_toml_table(table: Mapping[str, Any], prefix: tuple[str, ...] = ()) -> str:
    """Render a TOML table using only stdlib features."""
    lines: list[str] = []
    child_tables: list[tuple[tuple[str, ...], Mapping[str, Any]]] = []

    for key, value in table.items():
        if isinstance(value, Mapping):
            child_tables.append(((*prefix, key), value))
        else:
            lines.append(f"{key} = {_format_toml_value(value)}")

    rendered_sections: list[str] = []
    if prefix:
        header = ".".join(prefix)
        body = "\n".join(lines)
        rendered_sections.append(f"[{header}]\n{body}" if body else f"[{header}]")
    elif lines:
        rendered_sections.append("\n".join(lines))

    for child_prefix, child_table in child_tables:
        rendered_sections.append(_render_toml_table(child_table, child_prefix))

    return "\n\n".join(section for section in rendered_sections if section) + "\n"


def _format_toml_value(value: Any) -> str:
    """Render a scalar TOML value."""
    if isinstance(value, bool):
        return "true" if value else "false"
    if isinstance(value, (int, float)):
        return str(value)
    if isinstance(value, str):
        escaped = value.replace("\\", "\\\\").replace('"', '\\"')
        return f'"{escaped}"'
    if isinstance(value, list):
        inner = ", ".join(_format_toml_value(item) for item in value)
        return f"[{inner}]"
    raise TypeError(f"Unsupported TOML value: {value!r}")


def _deep_merge(target: dict[str, Any], source: dict[str, Any]) -> None:
    """Recursively merge *source* into *target* in place.

    Thin wrapper kept for backward compatibility; delegates to
    :func:`dekk._compat.deep_merge`.
    """
    merged = deep_merge(target, source)
    target.clear()
    target.update(merged)



"""Tests for platform detection."""

import platform

from dekk.detection.detect import PlatformDetector, PlatformInfo


def test_platform_detector_always_succeeds():
    """Platform detection should never raise."""
    detector = PlatformDetector()
    info = detector.detect()
    assert isinstance(info, PlatformInfo)
    assert info.os in ("Linux", "Darwin", "Windows")


def test_platform_properties():
    """Test platform property helpers."""
    linux_info = PlatformInfo(os="Linux", arch="x86_64")
    assert linux_info.is_linux
    assert not linux_info.is_macos
    assert not linux_info.is_windows

    macos_info = PlatformInfo(os="Darwin", arch="arm64")
    assert not macos_info.is_linux
    assert macos_info.is_macos
    assert not macos_info.is_windows


def test_current_platform_detection():
    """Test detection on current platform."""
    detector = PlatformDetector()
    info = detector.detect()

    # Should match Python's platform module
    assert info.os == platform.system()
    assert info.arch in ("x86_64", "aarch64", "arm64")



"""Tests for the ``python -m dekk`` entry point."""

from __future__ import annotations

import runpy
from unittest.mock import patch

import pytest

from dekk.cli.errors import ExitCodes, NotFoundError
from dekk.cli.main import main


def test_python_m_dekk_delegates_to_cli_main() -> None:
    with patch("dekk.cli.main.main") as mock_main:
        runpy.run_module("dekk", run_name="__main__")

    mock_main.assert_called_once_with()


def test_main_formats_dekk_error_for_users() -> None:
    with (
        patch("sys.argv", ["dekk"]),
        patch("dekk.cli.main._app", None),
        patch(
            "dekk.cli.main._make_app",
            side_effect=NotFoundError("Missing config", hint="Run 'dekk init'"),
        ),
        patch("dekk.cli.styles.print_error") as mock_error,
        patch("dekk.cli.styles.print_info") as mock_info,
    ):
        with pytest.raises(SystemExit) as exc_info:
            main()

    assert exc_info.value.code == int(ExitCodes.NOT_FOUND)
    mock_error.assert_called_once_with("Missing config")
    mock_info.assert_called_once_with("Hint: Run 'dekk init'")



"""Tests for dekk.cli.progress -- progress bars and spinners."""

from __future__ import annotations

from rich.progress import Progress

from dekk.cli.progress import progress_bar, spinner

# ---------------------------------------------------------------------------
# progress_bar
# ---------------------------------------------------------------------------


class TestProgressBar:
    """Tests for the progress_bar context manager."""

    def test_yields_progress_instance(self, capture_console):
        with capture_console():
            with progress_bar("Test", total=10) as p:
                assert isinstance(p, Progress)

    def test_task_auto_added(self, capture_console):
        with capture_console():
            with progress_bar("Building", total=5) as p:
                assert len(p.tasks) == 1
                assert p.tasks[0].description == "Building"
                assert p.tasks[0].total == 5

    def test_indeterminate_total(self, capture_console):
        with capture_console():
            with progress_bar("Loading", total=None) as p:
                assert p.tasks[0].total is None

    def test_advance_updates_completed(self, capture_console):
        with capture_console():
            with progress_bar("Work", total=3) as p:
                task = p.tasks[0]
                p.update(task.id, advance=1)
                assert task.completed == 1
                p.update(task.id, advance=2)
                assert task.completed == 3

    def test_context_manager_cleans_up(self, capture_console):
        """Progress should not raise on exit."""
        with capture_console():
            with progress_bar("Done", total=1) as p:
                p.update(p.tasks[0].id, advance=1)
            # No exception means cleanup succeeded


# ---------------------------------------------------------------------------
# spinner
# ---------------------------------------------------------------------------


class TestSpinner:
    """Tests for the spinner context manager."""

    def test_does_not_raise(self, capture_console):
        with capture_console():
            with spinner("Resolving..."):
                pass

    def test_yields_none(self, capture_console):
        with capture_console():
            with spinner("Working...") as result:
                assert result is None

    def test_code_runs_inside_block(self, capture_console):
        """Code within the spinner block should execute normally."""
        executed = False
        with capture_console():
            with spinner("Testing..."):
                executed = True
        assert executed



"""Enhanced Typer wrapper with dekk detection and optional Tully tracking.

Provides a drop-in replacement for ``typer.Typer`` that adds:

- Lazy-loaded execution context (platform, conda, CI, workspace)
- Session hooks (before/after each command)
- Optional experiment tracking via Tully
- Built-in doctor/version/env commands

The base ``pip install dekk`` install includes the CLI stack.

Typer is imported lazily -- this module can be imported without triggering
a typer/click/rich import chain.
"""

from __future__ import annotations

import functools
import importlib
from collections.abc import Callable
from pathlib import Path
from typing import Any, Final, cast

# Typer is loaded lazily on first use
_typer = None
_TYPER_AVAILABLE: bool | None = None


TYPER_EXPORTS: Final = ("Option", "Argument", "Exit", "Context")
TRACKING_STATUS_SUCCESS: Final = "success"
TRACKING_STATUS_FAILED: Final = "failed"
DEFAULT_APP_NAME: Final = "app"
AUTO_ACTIVATE_SPEC_NAME: Final = ".dekk.toml"
BUILTIN_DOCTOR_COMMAND: Final = "doctor"
BUILTIN_VERSION_COMMAND: Final = "version"
BUILTIN_ENV_COMMAND: Final = "env"
PREPEND_ENV_VARS: Final = {
    "PATH",
    "LD_LIBRARY_PATH",
    "DYLD_LIBRARY_PATH",
    "PYTHONPATH",
    "PKG_CONFIG_PATH",
}


def _get_typer() -> Any:
    """Import typer on first use."""
    global _typer, _TYPER_AVAILABLE
    if _TYPER_AVAILABLE is None:
        try:
            import typer as _t

            _typer = _t
            _TYPER_AVAILABLE = True
        except ImportError:
            _TYPER_AVAILABLE = False
    if not _TYPER_AVAILABLE:
        raise ImportError(
            "typer is required for dekk.Typer. "
            "Install or repair the package with: pip install --upgrade dekk"
        )
    return _typer


# Module-level __getattr__ for lazy Option, Argument, Exit access
def __getattr__(name: str) -> Any:  # noqa: N807
    if name in TYPER_EXPORTS:
        t = _get_typer()
        val = getattr(t, name)
        globals()[name] = val
        return val
    raise AttributeError(f"module 'dekk.cli.typer_app' has no attribute {name!r}")


class Typer:
    """Enhanced Typer with dekk detection and Tully integration.

    Uses composition to wrap ``typer.Typer`` without requiring typer
    to be imported at class definition time.
    """

    def __init__(
        self,
        *,
        name: str | None = None,
        enable_tracking: bool = False,
        tully_db_path: Path | None = None,
        tully_experiment_name: str | None = None,
        auto_capture_env: bool = True,
        auto_activate: bool = False,
        fail_fast: bool = True,
        add_doctor_command: bool = False,
        add_version_command: bool = False,
        add_env_command: bool = False,
        add_worktree_command: bool = False,
        add_agents_command: bool = False,
        project_version: str | None = None,
        **typer_kwargs: Any,
    ) -> None:
        t = _get_typer()
        self._app = t.Typer(**typer_kwargs)

        self._name = name
        self._enable_tracking = enable_tracking
        self._tully_db_path = tully_db_path
        self._tully_experiment_name = tully_experiment_name
        self._auto_capture_env = auto_capture_env
        self._auto_activate = auto_activate
        self._fail_fast = fail_fast
        self._project_version = project_version

        # Lazy-loaded state
        self._context: Any | None = None
        self._tully_client: Any | None = None
        self._current_run_id: str | None = None

        # Session hooks
        self._before_hooks: list[Callable[..., Any]] = []
        self._after_hooks: list[Callable[..., Any]] = []

        # Register auto-activation hook if enabled
        if auto_activate:
            self._before_hooks.append(self._auto_activation_hook)

        # Built-in commands
        if add_doctor_command:
            self._add_doctor_command()
        if add_version_command:
            self._add_version_command()
        if add_env_command:
            self._add_env_command()
        if add_worktree_command:
            self._add_worktree_command()
        if add_agents_command:
            self._add_agents_command()

    # -- Proxy to underlying typer.Typer for compatibility --------------------

    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        return self._app(*args, **kwargs)

    def __getattr__(self, name: str) -> Any:
        # Proxy attribute access to the underlying typer.Typer instance.
        # This is only called for attributes not found on self via __dict__.
        # Block dunder lookups to avoid infinite recursion, but allow
        # single-underscore attrs (e.g. typer's _add_completion) to proxy.
        if name.startswith("__") and name.endswith("__"):
            raise AttributeError(name)
        return getattr(self._app, name)

    # -- Lazy-loaded properties -----------------------------------------------

    @property
    def context(self) -> Any:
        """Execution context (lazy-loaded on first access)."""
        if self._context is None:
            from dekk.core.context import ExecutionContext

            self._context = ExecutionContext.capture()
        return self._context

    @property
    def platform(self) -> Any:
        """Shortcut to ``context.platform``."""
        return self.context.platform

    @property
    def conda_env(self) -> Any:
        """Shortcut to ``context.conda_env``."""
        return self.context.conda_env

    @property
    def ci_info(self) -> Any:
        """Shortcut to ``context.ci_info``."""
        return self.context.ci_info

    @property
    def workspace(self) -> Any:
        """Shortcut to ``context.workspace``."""
        return self.context.workspace

    # -- Session hooks --------------------------------------------------------

    def before_command(self, hook: Callable[..., Any]) -> None:
        """Register a hook to run before each command execution."""
        self._before_hooks.append(hook)

    def after_command(self, hook: Callable[..., Any]) -> None:
        """Register a hook to run after each command execution."""
        self._after_hooks.append(hook)

    def _auto_activation_hook(self, ctx: Any) -> None:
        """Auto-activate environment from .dekk.toml before each command."""
        import os

        from dekk.cli.errors import DependencyError, NotFoundError
        from dekk.cli.styles import print_error
        from dekk.environment.activation import EnvironmentActivator
        from dekk.environment.spec import find_envspec

        spec_file = find_envspec()
        if not spec_file:
            if self._fail_fast:
                raise NotFoundError(
                    f"No {AUTO_ACTIVATE_SPEC_NAME} found for auto-activation",
                    hint="Run 'dekk init' or set auto_activate=False",
                )
            return

        activator = EnvironmentActivator.from_cwd()
        result = activator.activate()

        if result.missing_tools:
            error_msg = f"Missing required dependencies: {', '.join(result.missing_tools)}"
            if self._fail_fast:
                app_name = self._name or DEFAULT_APP_NAME
                hint = (
                    f"Run '{app_name} {BUILTIN_DOCTOR_COMMAND}' to diagnose "
                    f"or '{app_name} install' to set up"
                )
                raise DependencyError(error_msg, hint=hint)
            else:
                print_error(error_msg)

        if result.env_vars:
            for key, value in result.env_vars.items():
                if key in PREPEND_ENV_VARS:
                    current = os.environ.get(key, "")
                    os.environ[key] = f"{value}{os.pathsep}{current}" if current else value
                else:
                    os.environ[key] = value

    # -- Enhanced command decorator -------------------------------------------

    def command(
        self,
        *args: Any,
        track: bool | None = None,
        capture_env: bool | None = None,
        catch_errors: bool = True,
        agent_skill: bool = False,
        **kwargs: Any,
    ) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Enhanced command decorator with optional tracking and error handling.

        Args:
            agent_skill: If True, marks this command for agent skill generation.
                When ``dekk agents init`` introspects this app, only commands
                with ``agent_skill=True`` are converted into SKILL.md templates.
        """
        should_track = track if track is not None else self._enable_tracking
        should_capture = capture_env if capture_env is not None else self._auto_capture_env

        base_decorator = self._app.command(*args, **kwargs)

        def enhanced_decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            if agent_skill:
                func._dekk_agent_skill = True  # type: ignore[attr-defined]

            @functools.wraps(func)
            def wrapper(*inner_args: Any, **inner_kwargs: Any) -> Any:
                ctx = self.context if should_capture else None

                for hook in self._before_hooks:
                    hook(ctx)

                run_id: str | None = None
                if should_track:
                    run_id = self._start_tracking(func.__name__, self.context)

                try:
                    result = func(*inner_args, **inner_kwargs)

                    if should_track and run_id:
                        self._complete_tracking(run_id, TRACKING_STATUS_SUCCESS)

                    return result
                except Exception as exc:
                    if should_track and run_id:
                        self._complete_tracking(run_id, TRACKING_STATUS_FAILED, error=str(exc))

                    if catch_errors:
                        from dekk.cli.errors import DekkError

                        if isinstance(exc, DekkError):
                            from dekk.cli.styles import print_error, print_info

                            print_error(exc.message)
                            if exc.hint:
                                print_info(f"Hint: {exc.hint}")
                            t = _get_typer()
                            raise t.Exit(exc.exit_code) from exc

                    raise
                finally:
                    after_ctx = self.context if should_capture else None
                    for hook in self._after_hooks:
                        hook(after_ctx)

            decorated = cast(Callable[..., Any], base_decorator(wrapper))
            return decorated

        return enhanced_decorator

    # -- Tracking helpers -----------------------------------------------------

    def log_metric(self, name: str, value: float, step: int | None = None) -> None:
        """Log a metric to Tully (if tracking is enabled and a run is active)."""
        if self._enable_tracking and self._current_run_id:
            client = self._get_tully_client()
            if client is not None:
                client.log_metric(self._current_run_id, name, value, step)

    def log_artifact(self, name: str, path: Path) -> None:
        """Log an artifact to Tully (if tracking is enabled and a run is active)."""
        if self._enable_tracking and self._current_run_id:
            client = self._get_tully_client()
            if client is not None:
                client.log_artifact(self._current_run_id, name, path)

    def _get_tully_client(self) -> Any | None:
        """Get or create the Tully client."""
        if self._tully_client is None:
            try:
                tully_module = importlib.import_module("tully")
                TullyClient = tully_module.TullyClient
                self._tully_client = TullyClient(db_path=self._tully_db_path)
            except ImportError:
                return None
        return self._tully_client

    def _start_tracking(self, command_name: str, context: Any) -> str | None:
        """Start a Tully tracking run."""
        client = self._get_tully_client()
        if client is None:
            return None

        run_id = client.start_run(
            command_name=command_name,
            experiment_name=self._tully_experiment_name,
            environment=context.to_dict(),
        )
        run_id_str = str(run_id)
        self._current_run_id = run_id_str
        return run_id_str

    def _complete_tracking(self, run_id: str, status: str, error: str | None = None) -> None:
        """Complete a Tully tracking run."""
        client = self._get_tully_client()
        if client is not None:
            client.complete_run(run_id, status, error=error)
        self._current_run_id = None

    # -- Built-in commands ----------------------------------------------------

    def _add_doctor_command(self) -> None:
        """Register the built-in ``doctor`` command."""

        @self.command(name=BUILTIN_DOCTOR_COMMAND)
        def doctor() -> None:
            """Check system environment and dependencies."""
            from dekk.cli.cli_commands import run_doctor

            run_doctor(self.context)

    def _add_version_command(self) -> None:
        """Register the built-in ``version`` command."""
        app_name = self._name
        version = self._project_version

        @self.command(name=BUILTIN_VERSION_COMMAND)
        def version_cmd() -> None:
            """Show version information."""
            from dekk.cli.cli_commands import run_version

            run_version(app_name, version, self.context)

    def _add_env_command(self) -> None:
        """Register the built-in ``env`` command."""

        @self.command(name=BUILTIN_ENV_COMMAND)
        def env() -> None:
            """Show environment information."""
            from dekk.cli.cli_commands import run_env

            run_env(self.context)

    def _add_worktree_command(self) -> None:
        """Register the built-in ``worktree`` sub-app."""
        from dekk.tools.worktree import create_worktree_app

        self.add_typer(create_worktree_app(), name="worktree")

    def _add_agents_command(self) -> None:
        """Register the built-in ``agents`` sub-app."""
        from dekk.agents import create_agents_app

        self.add_typer(create_agents_app(parent_app=self._app), name="agents")



"""Tests for dekk.cli.main command wiring."""

from __future__ import annotations

from unittest.mock import patch

import pytest

import dekk.cli.main as cli_main


class TestCliMain:
    def test_main_routes_unknown_first_arg_to_project_runner(self, monkeypatch):
        monkeypatch.setattr(cli_main, "_app", None)
        monkeypatch.setattr("sys.argv", ["dekk", "demo", "hello"])
        with patch("dekk.project.runner.run_project_command", return_value=0) as run_mock:
            with pytest.raises(SystemExit) as exc:
                cli_main.main()
        assert exc.value.code == 0
        run_mock.assert_called_once_with("demo", ["hello"])

    def test_agents_routes_to_project_runner(self, monkeypatch):
        """``dekk agents`` is no longer a builtin — it routes through the project runner."""
        monkeypatch.setattr(cli_main, "_app", None)
        monkeypatch.setattr("sys.argv", ["dekk", "agents", "init"])
        with patch("dekk.project.runner.run_project_command", return_value=0) as run_mock:
            with pytest.raises(SystemExit) as exc:
                cli_main.main()
        assert exc.value.code == 0
        run_mock.assert_called_once_with("agents", ["init"])

    def test_worktree_routes_to_project_runner(self, monkeypatch):
        """``dekk worktree`` is no longer a builtin — it routes through the project runner."""
        monkeypatch.setattr(cli_main, "_app", None)
        monkeypatch.setattr("sys.argv", ["dekk", "worktree", "list"])
        with patch("dekk.project.runner.run_project_command", return_value=0) as run_mock:
            with pytest.raises(SystemExit) as exc:
                cli_main.main()
        assert exc.value.code == 0
        run_mock.assert_called_once_with("worktree", ["list"])



"""Conda environment detection."""

from __future__ import annotations

import json
import os
import subprocess
from dataclasses import dataclass
from pathlib import Path

# Common conda installation paths (relative to $HOME, except /opt/conda).
# Used as fallback when conda/mamba is not on PATH.
COMMON_INSTALL_PATHS: tuple[str, ...] = (
    "miniforge3",
    "mambaforge",
    "miniconda3",
    "anaconda3",
)


@dataclass(frozen=True)
class CondaEnvironment:
    """Conda environment information."""

    name: str
    prefix: Path
    is_active: bool = False
    python_version: str | None = None


@dataclass(frozen=True)
class CondaValidation:
    """Result of validating a conda environment."""

    env_name: str
    found: bool
    prefix: Path | None = None
    is_active: bool = False
    missing_packages: tuple[str, ...] = ()
    errors: tuple[str, ...] = ()

    @property
    def ok(self) -> bool:
        """True when the environment exists and has all required packages."""
        return self.found and len(self.missing_packages) == 0 and len(self.errors) == 0


class CondaDetector:
    """Detect conda/mamba environments."""

    def __init__(self, timeout: float = 10.0):
        """
        Initialize conda detector.

        Args:
            timeout: Timeout for conda commands.
        """
        self.timeout = timeout

    def find_active(self) -> CondaEnvironment | None:
        """
        Find the currently active conda environment.

        Returns:
            CondaEnvironment if one is active, None otherwise.
        """
        conda_prefix = os.environ.get("CONDA_PREFIX")
        conda_name = os.environ.get("CONDA_DEFAULT_ENV")

        if not conda_prefix:
            return None

        prefix = Path(conda_prefix)
        name = conda_name or prefix.name

        # Try to get Python version
        python_version = self._get_python_version(prefix)

        return CondaEnvironment(
            name=name, prefix=prefix, is_active=True, python_version=python_version
        )

    def find_environment(self, name: str) -> CondaEnvironment | None:
        """
        Find a conda environment by name.

        Args:
            name: Environment name to search for.

        Returns:
            CondaEnvironment if found, None otherwise.
        """
        # Check if conda is available
        import shutil

        conda_cmd = shutil.which("conda") or shutil.which("mamba")
        if not conda_cmd:
            return None

        try:
            # Get list of environments
            result = subprocess.run(
                [conda_cmd, "env", "list", "--json"],
                capture_output=True,
                text=True,
                timeout=self.timeout,
                check=False,
            )

            if result.returncode != 0:
                return None

            data = json.loads(result.stdout)
            envs = data.get("envs", [])

            # Find matching environment
            for env_path in envs:
                env_path = Path(env_path)
                if env_path.name == name:
                    # Check if this is the active environment
                    active_prefix = os.environ.get("CONDA_PREFIX")
                    is_active = str(env_path) == active_prefix

                    python_version = self._get_python_version(env_path)

                    return CondaEnvironment(
                        name=name,
                        prefix=env_path,
                        is_active=is_active,
                        python_version=python_version,
                    )

            return None

        except (subprocess.TimeoutExpired, json.JSONDecodeError, OSError):
            return None

    def find_prefix(self, env_name: str, *, probe_common: bool = True) -> Path | None:
        """Find a conda environment prefix by name.

        Searches in order:
        1. Currently active environment (if its name matches *env_name*).
        2. ``conda env list --json`` / ``mamba env list --json``.
        3. Common installation paths under ``$HOME`` and ``/opt/conda``
           (only when *probe_common* is True).

        Args:
            env_name: Environment name to look for.
            probe_common: When True, probe well-known filesystem paths as a
                fallback when conda/mamba is not on PATH.

        Returns:
            Path to the environment prefix, or None if not found.
        """
        # 1. Active environment
        active = self.find_active()
        if active is not None and active.name == env_name:
            return active.prefix

        # 2. Query conda/mamba
        env = self.find_environment(env_name)
        if env is not None:
            return env.prefix

        # 3. Probe common paths
        if probe_common:
            for candidate in self._common_prefix_candidates(env_name):
                try:
                    if candidate.is_dir():
                        return candidate
                except OSError:
                    continue

        return None

    def validate(
        self,
        env_name: str,
        *,
        required_packages: list[str] | None = None,
    ) -> CondaValidation:
        """Validate that a conda environment exists and contains required packages.

        Args:
            env_name: Environment name to validate.
            required_packages: Package names that must be installed in the
                environment (checked via ``conda list --json``).

        Returns:
            A CondaValidation result.
        """
        prefix = self.find_prefix(env_name)
        if prefix is None:
            return CondaValidation(
                env_name=env_name,
                found=False,
                errors=(f"Environment '{env_name}' not found",),
            )

        active = self.find_active()
        is_active = active is not None and active.prefix == prefix

        if not required_packages:
            return CondaValidation(
                env_name=env_name,
                found=True,
                prefix=prefix,
                is_active=is_active,
            )

        # Check installed packages
        missing = self._check_packages(prefix, required_packages)

        return CondaValidation(
            env_name=env_name,
            found=True,
            prefix=prefix,
            is_active=is_active,
            missing_packages=tuple(missing),
        )

    # ------------------------------------------------------------------
    # Private helpers
    # ------------------------------------------------------------------

    @staticmethod
    def _common_prefix_candidates(env_name: str) -> list[Path]:
        """Build a list of candidate paths for *env_name* under common install locations."""
        home = Path.home()
        candidates = [home / base / "envs" / env_name for base in COMMON_INSTALL_PATHS]
        candidates.append(Path("/opt/conda/envs") / env_name)
        return candidates

    def _check_packages(self, prefix: Path, packages: list[str]) -> list[str]:
        """Return names from *packages* that are NOT installed in *prefix*."""
        import shutil

        conda_cmd = shutil.which("conda") or shutil.which("mamba")
        if not conda_cmd:
            # Cannot verify -- assume all missing
            return list(packages)

        try:
            result = subprocess.run(
                [conda_cmd, "list", "--prefix", str(prefix), "--json"],
                capture_output=True,
                text=True,
                timeout=self.timeout,
                check=False,
            )
            if result.returncode != 0:
                return list(packages)

            installed = {pkg["name"] for pkg in json.loads(result.stdout)}
            return [p for p in packages if p not in installed]
        except (subprocess.TimeoutExpired, json.JSONDecodeError, OSError, KeyError):
            return list(packages)

    def _get_python_version(self, prefix: Path) -> str | None:
        """Get Python version in a conda environment."""
        # Try to find python executable
        if os.name == "nt":  # Windows
            python_path = prefix / "python.exe"
        else:  # Unix
            python_path = prefix / "bin" / "python"

        if not python_path.exists():
            return None

        try:
            result = subprocess.run(
                [str(python_path), "--version"],
                capture_output=True,
                text=True,
                timeout=5.0,
                check=False,
            )

            output = result.stdout + result.stderr
            # Parse "Python X.Y.Z"
            import re

            match = re.search(r"Python\s+(\d+\.\d+\.\d+)", output)
            if match:
                return match.group(1)

            return None

        except (subprocess.TimeoutExpired, OSError):
            return None



"""Tests for version manager detection."""

import pytest

from dekk.detection.version_managers import (
    ManagedVersion,
    VersionManagerDetector,
    VersionManagerInfo,
)


@pytest.fixture
def detector():
    return VersionManagerDetector()


class TestVersionManagerInfo:
    def test_is_available_true(self, tmp_path):
        info = VersionManagerInfo(
            name="test",
            command="test",
            root=tmp_path,
        )
        assert info.is_available is True

    def test_is_available_false(self, tmp_path):
        info = VersionManagerInfo(
            name="test",
            command="test",
            root=tmp_path / "no_such_dir",
        )
        assert info.is_available is False

    def test_version_count(self, tmp_path):
        versions = (
            ManagedVersion("3.11.0", tmp_path / "3.11.0"),
            ManagedVersion("3.12.0", tmp_path / "3.12.0"),
        )
        info = VersionManagerInfo(
            name="test",
            command="test",
            root=tmp_path,
            installed_versions=versions,
        )
        assert info.version_count == 2


class TestManagedVersion:
    def test_creation(self, tmp_path):
        mv = ManagedVersion("3.11.0", tmp_path, is_active=True)
        assert mv.version == "3.11.0"
        assert mv.path == tmp_path
        assert mv.is_active is True

    def test_default_not_active(self, tmp_path):
        mv = ManagedVersion("3.11.0", tmp_path)
        assert mv.is_active is False


class TestPyenvDetection:
    def test_detects_pyenv(self, tmp_path, monkeypatch, detector):
        # Set up a fake pyenv structure
        pyenv_root = tmp_path / ".pyenv"
        versions_dir = pyenv_root / "versions"
        (versions_dir / "3.11.0").mkdir(parents=True)
        (versions_dir / "3.12.0").mkdir(parents=True)

        monkeypatch.setenv("PYENV_ROOT", str(pyenv_root))
        monkeypatch.setenv("PYENV_VERSION", "3.12.0")

        info = detector.detect("pyenv")
        assert info is not None
        assert info.name == "pyenv"
        assert info.version_count == 2
        assert info.active_version == "3.12.0"

        # Check active flag
        active = [v for v in info.installed_versions if v.is_active]
        assert len(active) == 1
        assert active[0].version == "3.12.0"

    def test_no_pyenv(self, tmp_path, monkeypatch, detector):
        monkeypatch.setenv("PYENV_ROOT", str(tmp_path / "nonexistent"))
        info = detector.detect("pyenv")
        assert info is None


class TestRustupDetection:
    def test_detects_rustup(self, tmp_path, monkeypatch, detector):
        rustup_home = tmp_path / ".rustup"
        toolchains = rustup_home / "toolchains"
        (toolchains / "stable-x86_64-unknown-linux-gnu").mkdir(parents=True)
        (toolchains / "nightly-x86_64-unknown-linux-gnu").mkdir(parents=True)

        # Write a settings.toml
        settings = rustup_home / "settings.toml"
        settings.write_text('default_toolchain = "nightly-x86_64-unknown-linux-gnu"\n')

        monkeypatch.setenv("RUSTUP_HOME", str(rustup_home))
        monkeypatch.delenv("RUSTUP_TOOLCHAIN", raising=False)

        info = detector.detect("rustup")
        assert info is not None
        assert info.name == "rustup"
        assert info.version_count == 2
        assert info.active_version == "nightly-x86_64-unknown-linux-gnu"

    def test_rustup_env_override(self, tmp_path, monkeypatch, detector):
        rustup_home = tmp_path / ".rustup"
        (rustup_home / "toolchains" / "stable").mkdir(parents=True)

        monkeypatch.setenv("RUSTUP_HOME", str(rustup_home))
        monkeypatch.setenv("RUSTUP_TOOLCHAIN", "stable")

        info = detector.detect("rustup")
        assert info is not None
        assert info.active_version == "stable"


class TestNvmDetection:
    def test_detects_nvm(self, tmp_path, monkeypatch, detector):
        nvm_dir = tmp_path / ".nvm"
        node_versions = nvm_dir / "versions" / "node"
        (node_versions / "v18.17.0").mkdir(parents=True)
        (node_versions / "v20.10.0").mkdir(parents=True)

        monkeypatch.setenv("NVM_DIR", str(nvm_dir))
        monkeypatch.setenv("NVM_BIN", str(node_versions / "v20.10.0" / "bin"))

        info = detector.detect("nvm")
        assert info is not None
        assert info.name == "nvm"
        assert info.version_count == 2
        assert info.active_version == "20.10.0"

    def test_no_nvm(self, tmp_path, monkeypatch, detector):
        monkeypatch.setenv("NVM_DIR", str(tmp_path / "nonexistent"))
        info = detector.detect("nvm")
        assert info is None


class TestRbenvDetection:
    def test_detects_rbenv(self, tmp_path, monkeypatch, detector):
        rbenv_root = tmp_path / ".rbenv"
        versions = rbenv_root / "versions"
        (versions / "3.2.0").mkdir(parents=True)
        (versions / "3.3.0").mkdir(parents=True)

        monkeypatch.setenv("RBENV_ROOT", str(rbenv_root))
        monkeypatch.setenv("RBENV_VERSION", "3.3.0")

        info = detector.detect("rbenv")
        assert info is not None
        assert info.name == "rbenv"
        assert info.version_count == 2
        assert info.active_version == "3.3.0"


class TestAsdfDetection:
    def test_detects_asdf(self, tmp_path, monkeypatch, detector):
        asdf_dir = tmp_path / ".asdf"
        installs = asdf_dir / "installs"
        (installs / "python" / "3.11.0").mkdir(parents=True)
        (installs / "nodejs" / "20.10.0").mkdir(parents=True)

        monkeypatch.setenv("ASDF_DATA_DIR", str(asdf_dir))

        info = detector.detect("asdf")
        assert info is not None
        assert info.name == "asdf"
        assert info.version_count == 2
        assert any("python/3.11.0" in v.version for v in info.installed_versions)
        assert any("nodejs/20.10.0" in v.version for v in info.installed_versions)


class TestSdkmanDetection:
    def test_detects_sdkman(self, tmp_path, monkeypatch, detector):
        sdkman_dir = tmp_path / ".sdkman"
        candidates = sdkman_dir / "candidates"
        (candidates / "java" / "21.0.1-tem").mkdir(parents=True)
        (candidates / "java" / "17.0.9-tem").mkdir(parents=True)

        monkeypatch.setenv("SDKMAN_DIR", str(sdkman_dir))

        info = detector.detect("sdkman")
        assert info is not None
        assert info.name == "sdkman"
        assert info.version_count == 2


class TestDetectAll:
    def test_detect_all_returns_list(self, tmp_path, monkeypatch, detector):
        # Set up pyenv
        pyenv_root = tmp_path / ".pyenv"
        (pyenv_root / "versions" / "3.11.0").mkdir(parents=True)
        monkeypatch.setenv("PYENV_ROOT", str(pyenv_root))

        # Ensure others are absent
        monkeypatch.setenv("NVM_DIR", str(tmp_path / "nonexistent"))
        monkeypatch.setenv("RBENV_ROOT", str(tmp_path / "nonexistent"))
        monkeypatch.setenv("RUSTUP_HOME", str(tmp_path / "nonexistent"))
        monkeypatch.setenv("GOENV_ROOT", str(tmp_path / "nonexistent"))
        monkeypatch.setenv("SDKMAN_DIR", str(tmp_path / "nonexistent"))
        monkeypatch.setenv("ASDF_DATA_DIR", str(tmp_path / "nonexistent"))

        results = detector.detect_all()
        assert isinstance(results, list)
        # At least pyenv should be detected
        names = [r.name for r in results]
        assert "pyenv" in names

    def test_detect_unknown_returns_none(self, detector):
        assert detector.detect("unknown_manager") is None



"""Tests for configuration management."""

from __future__ import annotations

from pathlib import Path

import pytest

from dekk.core.config import ConfigManager, ConfigReconciler, ConfigSource
from dekk.core.paths import project_config_file, user_config_file

# ===========================================================================
# ConfigSource tests
# ===========================================================================


class TestConfigSource:
    def test_basic_creation(self):
        src = ConfigSource("key", "value", "default", None, None, 0)
        assert src.key == "key"
        assert src.value == "value"
        assert src.source == "default"
        assert src.file_path is None
        assert src.line_number is None
        assert src.precedence == 0

    def test_file_source(self):
        path = Path("/etc/app/config.toml")
        src = ConfigSource("db.host", "localhost", "file", path, 42, 1)
        assert src.file_path == path
        assert src.line_number == 42
        assert src.source == "file"

    def test_frozen(self):
        src = ConfigSource("key", "val", "default", None, None, 0)
        with pytest.raises(AttributeError):
            src.key = "other"  # type: ignore[misc]

    def test_frozen_value(self):
        src = ConfigSource("key", "val", "default", None, None, 0)
        with pytest.raises(AttributeError):
            src.value = "changed"  # type: ignore[misc]

    def test_frozen_precedence(self):
        src = ConfigSource("key", "val", "default", None, None, 0)
        with pytest.raises(AttributeError):
            src.precedence = 99  # type: ignore[misc]

    def test_equality(self):
        a = ConfigSource("k", "v", "default", None, None, 0)
        b = ConfigSource("k", "v", "default", None, None, 0)
        assert a == b

    def test_inequality_key(self):
        a = ConfigSource("k1", "v", "default", None, None, 0)
        b = ConfigSource("k2", "v", "default", None, None, 0)
        assert a != b

    def test_inequality_value(self):
        a = ConfigSource("k", "v1", "default", None, None, 0)
        b = ConfigSource("k", "v2", "default", None, None, 0)
        assert a != b

    def test_inequality_precedence(self):
        a = ConfigSource("k", "v", "default", None, None, 0)
        b = ConfigSource("k", "v", "default", None, None, 1)
        assert a != b

    def test_various_value_types_int(self):
        src = ConfigSource("count", 42, "default", None, None, 0)
        assert src.value == 42

    def test_various_value_types_float(self):
        src = ConfigSource("rate", 3.14, "default", None, None, 0)
        assert src.value == 3.14

    def test_various_value_types_bool(self):
        src = ConfigSource("enabled", True, "cli", None, None, 3)
        assert src.value is True

    def test_various_value_types_list(self):
        src = ConfigSource("items", [1, 2, 3], "file", Path("x.toml"), 1, 1)
        assert src.value == [1, 2, 3]

    def test_various_value_types_dict(self):
        src = ConfigSource("nested", {"a": 1}, "default", None, None, 0)
        assert src.value == {"a": 1}

    def test_various_value_types_none(self):
        src = ConfigSource("empty", None, "default", None, None, 0)
        assert src.value is None

    def test_source_types(self):
        for source_type in ("environment", "file", "default", "cli"):
            src = ConfigSource("k", "v", source_type, None, None, 0)
            assert src.source == source_type

    def test_hash(self):
        src = ConfigSource("k", "v", "default", None, None, 0)
        # Frozen dataclasses are hashable
        assert hash(src) is not None
        d = {src: True}
        assert d[src] is True

    def test_repr(self):
        src = ConfigSource("k", "v", "default", None, None, 0)
        r = repr(src)
        assert "ConfigSource" in r
        assert "k" in r


# ===========================================================================
# ConfigReconciler tests
# ===========================================================================


class TestConfigReconcilerInit:
    def test_empty(self):
        rec = ConfigReconciler()
        assert rec.sources == {}

    def test_keys_empty(self):
        rec = ConfigReconciler()
        assert rec.keys() == []


class TestConfigReconcilerAddSource:
    def test_add_single(self):
        rec = ConfigReconciler()
        src = ConfigSource("key", "val", "default", None, None, 0)
        rec.add_source(src)
        assert "key" in rec.sources
        assert len(rec.sources["key"]) == 1

    def test_add_multiple_same_key(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "a", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "b", "file", Path("f"), 1, 1))
        rec.add_source(ConfigSource("k", "c", "cli", None, None, 3))
        assert len(rec.sources["k"]) == 3

    def test_add_multiple_different_keys(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("x", 1, "default", None, None, 0))
        rec.add_source(ConfigSource("y", 2, "default", None, None, 0))
        assert "x" in rec.sources
        assert "y" in rec.sources

    def test_add_duplicate_source(self):
        rec = ConfigReconciler()
        src = ConfigSource("k", "v", "default", None, None, 0)
        rec.add_source(src)
        rec.add_source(src)
        assert len(rec.sources["k"]) == 2


class TestConfigReconcilerResolve:
    def test_resolve_single(self):
        rec = ConfigReconciler()
        src = ConfigSource("k", "v", "default", None, None, 0)
        rec.add_source(src)
        assert rec.resolve("k") == src

    def test_resolve_highest_precedence(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "low", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "mid", "file", Path("f"), 1, 1))
        rec.add_source(ConfigSource("k", "high", "cli", None, None, 3))
        result = rec.resolve("k")
        assert result.value == "high"
        assert result.precedence == 3

    def test_resolve_not_found(self):
        rec = ConfigReconciler()
        assert rec.resolve("missing") is None

    def test_resolve_env_over_file(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "file_val", "file", Path("f"), 1, 1))
        rec.add_source(ConfigSource("k", "env_val", "environment", None, None, 2))
        result = rec.resolve("k")
        assert result.value == "env_val"

    def test_resolve_cli_over_env(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "env_val", "environment", None, None, 2))
        rec.add_source(ConfigSource("k", "cli_val", "cli", None, None, 3))
        result = rec.resolve("k")
        assert result.value == "cli_val"

    def test_resolve_same_precedence(self):
        """When precedence is equal, highest precedence still wins (deterministic)."""
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "first", "file", Path("a"), 1, 1))
        rec.add_source(ConfigSource("k", "second", "file", Path("b"), 2, 1))
        result = rec.resolve("k")
        # Both have precedence 1, result is deterministic via max()
        assert result.precedence == 1
        assert result.value in ("first", "second")

    def test_resolve_negative_precedence(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "neg", "default", None, None, -1))
        rec.add_source(ConfigSource("k", "zero", "default", None, None, 0))
        result = rec.resolve("k")
        assert result.value == "zero"

    def test_resolve_large_precedence(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "normal", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "override", "cli", None, None, 9999))
        result = rec.resolve("k")
        assert result.value == "override"

    def test_resolve_returns_config_source(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "default", None, None, 0))
        result = rec.resolve("k")
        assert isinstance(result, ConfigSource)

    def test_resolve_preserves_file_path(self):
        rec = ConfigReconciler()
        path = Path("/etc/app/config.toml")
        rec.add_source(ConfigSource("k", "v", "file", path, 42, 10))
        result = rec.resolve("k")
        assert result.file_path == path
        assert result.line_number == 42


class TestConfigReconcilerExplain:
    def test_explain_not_found(self):
        rec = ConfigReconciler()
        assert rec.explain("missing") == "missing: not found"

    def test_explain_single_default(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "default", None, None, 0))
        text = rec.explain("k")
        assert "Configuration for 'k':" in text
        assert "default: v" in text
        assert "Final value: v (from default)" in text

    def test_explain_single_file(self):
        rec = ConfigReconciler()
        path = Path("config.yaml")
        rec.add_source(ConfigSource("k", "v", "file", path, 12, 1))
        text = rec.explain("k")
        assert "file: v (from config.yaml:12)" in text
        assert "Final value: v (from file)" in text

    def test_explain_multiple_sources(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("batch_size", 32, "default", None, None, 0))
        rec.add_source(ConfigSource("batch_size", 64, "file", Path("config.yaml"), 12, 1))
        rec.add_source(ConfigSource("batch_size", 128, "environment", None, None, 2))
        rec.add_source(ConfigSource("batch_size", 256, "cli", None, None, 3))
        text = rec.explain("batch_size")
        assert "Configuration for 'batch_size':" in text
        assert "default: 32" in text
        assert "file: 64 (from config.yaml:12)" in text
        assert "environment: 128" in text
        assert "cli: 256" in text
        assert "Final value: 256 (from cli)" in text

    def test_explain_sorted_by_precedence(self):
        rec = ConfigReconciler()
        # Add in reverse order
        rec.add_source(ConfigSource("k", "high", "cli", None, None, 3))
        rec.add_source(ConfigSource("k", "low", "default", None, None, 0))
        text = rec.explain("k")
        lines = text.split("\n")
        # default (prec 0) should appear before cli (prec 3)
        default_idx = next(i for i, line in enumerate(lines) if "default:" in line)
        cli_idx = next(i for i, line in enumerate(lines) if "cli:" in line)
        assert default_idx < cli_idx

    def test_explain_arrow_in_final_line(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "default", None, None, 0))
        text = rec.explain("k")
        assert "\u2192" in text  # Unicode arrow

    def test_explain_file_without_line_number(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "file", Path("f.toml"), None, 1))
        text = rec.explain("k")
        assert "file: v (from f.toml:None)" in text

    def test_explain_file_without_path(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "file", None, None, 1))
        text = rec.explain("k")
        # Without file_path, falls into the else branch
        assert "file: v" in text
        assert "(from" not in text.split("file: v")[1].split("\n")[0]


class TestConfigReconcilerKeys:
    def test_keys_sorted(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("z_key", 1, "default", None, None, 0))
        rec.add_source(ConfigSource("a_key", 2, "default", None, None, 0))
        rec.add_source(ConfigSource("m_key", 3, "default", None, None, 0))
        assert rec.keys() == ["a_key", "m_key", "z_key"]

    def test_keys_no_duplicates(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", 1, "default", None, None, 0))
        rec.add_source(ConfigSource("k", 2, "cli", None, None, 3))
        assert rec.keys() == ["k"]


class TestConfigReconcilerAllSources:
    def test_all_sources_not_found(self):
        rec = ConfigReconciler()
        assert rec.all_sources("missing") == []

    def test_all_sources_sorted_by_precedence(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "c", "cli", None, None, 3))
        rec.add_source(ConfigSource("k", "a", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "b", "file", Path("f"), 1, 1))
        sources = rec.all_sources("k")
        assert len(sources) == 3
        assert sources[0].precedence == 0
        assert sources[1].precedence == 1
        assert sources[2].precedence == 3

    def test_all_sources_returns_list(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "v", "default", None, None, 0))
        result = rec.all_sources("k")
        assert isinstance(result, list)
        assert all(isinstance(s, ConfigSource) for s in result)


# ===========================================================================
# Integration / full-workflow tests
# ===========================================================================


class TestConfigReconcilerIntegration:
    def test_full_precedence_chain(self):
        """default < file < environment < cli"""
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("host", "default.local", "default", None, None, 0))
        rec.add_source(ConfigSource("host", "file.local", "file", Path("app.toml"), 5, 1))
        rec.add_source(ConfigSource("host", "env.local", "environment", None, None, 2))
        rec.add_source(ConfigSource("host", "cli.local", "cli", None, None, 3))
        assert rec.resolve("host").value == "cli.local"

    def test_partial_chain_file_wins(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("port", 8080, "default", None, None, 0))
        rec.add_source(ConfigSource("port", 9090, "file", Path("cfg.toml"), 10, 1))
        assert rec.resolve("port").value == 9090

    def test_multiple_keys_independent(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("host", "a", "default", None, None, 0))
        rec.add_source(ConfigSource("host", "b", "cli", None, None, 3))
        rec.add_source(ConfigSource("port", 80, "default", None, None, 0))
        rec.add_source(ConfigSource("port", 443, "environment", None, None, 2))
        assert rec.resolve("host").value == "b"
        assert rec.resolve("port").value == 443

    def test_explain_after_resolve(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "a", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "b", "cli", None, None, 3))
        # resolve doesn't affect explain
        rec.resolve("k")
        text = rec.explain("k")
        assert "default: a" in text
        assert "cli: b" in text

    def test_add_source_after_resolve(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "low", "default", None, None, 0))
        assert rec.resolve("k").value == "low"
        rec.add_source(ConfigSource("k", "high", "cli", None, None, 3))
        assert rec.resolve("k").value == "high"

    def test_dotted_keys(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("database.host", "localhost", "default", None, None, 0))
        rec.add_source(ConfigSource("database.host", "prod.db", "file", Path("p.toml"), 7, 1))
        assert rec.resolve("database.host").value == "prod.db"

    def test_empty_string_key(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("", "val", "default", None, None, 0))
        assert rec.resolve("").value == "val"

    def test_empty_string_value(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "", "default", None, None, 0))
        assert rec.resolve("k").value == ""

    def test_many_sources_for_one_key(self):
        rec = ConfigReconciler()
        for i in range(20):
            rec.add_source(ConfigSource("k", f"v{i}", "default", None, None, i))
        result = rec.resolve("k")
        assert result.value == "v19"
        assert result.precedence == 19

    def test_explain_multiline_output(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "a", "default", None, None, 0))
        rec.add_source(ConfigSource("k", "b", "cli", None, None, 3))
        text = rec.explain("k")
        lines = text.strip().split("\n")
        assert len(lines) == 4  # header + 2 sources + final

    def test_keys_reflects_all_added(self):
        rec = ConfigReconciler()
        for name in ["z", "a", "m", "b"]:
            rec.add_source(ConfigSource(name, name, "default", None, None, 0))
        assert rec.keys() == ["a", "b", "m", "z"]

    def test_file_path_as_absolute(self):
        import sys

        rec = ConfigReconciler()
        path = (
            Path("C:/home/user/.config/app/config.toml")
            if sys.platform == "win32"
            else Path("/home/user/.config/app/config.toml")
        )
        rec.add_source(ConfigSource("k", "v", "file", path, 100, 1))
        result = rec.resolve("k")
        assert result.file_path == path
        assert result.file_path.is_absolute()

    def test_resolve_with_string_and_int_values(self):
        rec = ConfigReconciler()
        rec.add_source(ConfigSource("k", "string_val", "default", None, None, 0))
        rec.add_source(ConfigSource("k", 42, "cli", None, None, 3))
        result = rec.resolve("k")
        assert result.value == 42


# ===========================================================================
# ConfigManager tests (existing functionality)
# ===========================================================================


class TestConfigManager:
    def test_get_set(self):
        config = ConfigManager("test", defaults={"database": {"path": "/tmp/db"}})
        assert config.get("database.path") == "/tmp/db"
        assert config.get("nonexistent.key", default="fallback") == "fallback"
        config.set("custom.value", "test")
        assert config.get("custom.value") == "test"

    def test_env_override(self, monkeypatch):
        config = ConfigManager("testapp", env_prefix="TESTAPP", defaults={"key": "default"})
        monkeypatch.setenv("TESTAPP_KEY", "from_env")
        config._load()
        assert config.get("key") == "from_env"

    def test_to_dict(self):
        config = ConfigManager("test", defaults={"a": 1, "b": {"c": 2}})
        result = config.to_dict()
        assert result == {"a": 1, "b": {"c": 2}}

    def test_default_values(self):
        config = ConfigManager("test", defaults={"x": 10, "y": 20})
        assert config.get("x") == 10
        assert config.get("y") == 20

    def test_nested_set(self):
        config = ConfigManager("test", defaults={})
        config.set("a.b.c", "deep")
        assert config.get("a.b.c") == "deep"

    def test_get_missing_returns_none(self):
        config = ConfigManager("test", defaults={})
        assert config.get("nonexistent") is None

    def test_get_missing_returns_default(self):
        config = ConfigManager("test", defaults={})
        assert config.get("nonexistent", "fallback") == "fallback"

    def test_overwrite_set(self):
        config = ConfigManager("test", defaults={"k": "old"})
        config.set("k", "new")
        assert config.get("k") == "new"

    def test_deep_merge(self):
        config = ConfigManager("test", defaults={"a": {"b": 1, "c": 2}})
        config._merge({"a": {"c": 3, "d": 4}})
        assert config.get("a.b") == 1
        assert config.get("a.c") == 3
        assert config.get("a.d") == 4

    def test_env_prefix_default(self):
        config = ConfigManager("myapp")
        assert config.env_prefix == "MYAPP"

    def test_config_dir_default(self):
        config = ConfigManager("myapp")
        assert config.config_dir == ".myapp"

    def test_custom_env_prefix(self):
        config = ConfigManager("myapp", env_prefix="CUSTOM")
        assert config.env_prefix == "CUSTOM"

    def test_custom_config_dir(self):
        config = ConfigManager("myapp", config_dir=".custom")
        assert config.config_dir == ".custom"

    def test_loads_user_and_project_config_from_shared_path_policy(self, tmp_path, monkeypatch):
        home = tmp_path / "home"
        home.mkdir()
        project = tmp_path / "project"
        child = project / "subdir"
        child.mkdir(parents=True)
        monkeypatch.setenv("HOME", str(home))
        monkeypatch.chdir(child)

        user_path = user_config_file("testapp")
        user_path.parent.mkdir(parents=True, exist_ok=True)
        user_path.write_text('[database]\nhost = "user"\n', encoding="utf-8")

        project_path = project_config_file("testapp", start_dir=project)
        project_path.parent.mkdir(exist_ok=True)
        project_path.write_text('[database]\nhost = "project"\n', encoding="utf-8")

        config = ConfigManager("testapp")
        assert config.get("database.host") == "project"



"""GitHub Copilot provider implementation."""

from __future__ import annotations

from pathlib import Path

from dekk.agents.constants import (
    COPILOT_DIR,
    COPILOT_INSTRUCTIONS,
    COPILOT_PER_DIR,
    COPILOT_RULE_SUFFIX,
    TARGET_COPILOT,
)
from dekk.agents.discovery import RuleDefinition
from dekk.agents.providers.base import AgentContext, DekkAgent
from dekk.agents.providers.shared import remove_dir_if_empty, remove_file, remove_tree


def generate_copilot_per_directory(project_root: Path, rules: list[RuleDefinition]) -> None:
    """Generate `.github/instructions/` from rules with Copilot `applyTo:` frontmatter."""
    instr_dir = project_root / COPILOT_DIR / COPILOT_PER_DIR
    instr_dir.mkdir(parents=True, exist_ok=True)
    for rule in rules:
        apply_to = ",".join(rule.paths)
        content = f"---\napplyTo: {apply_to}\n---\n{rule.body}"
        (instr_dir / f"{rule.name}{COPILOT_RULE_SUFFIX}").write_text(content, encoding="utf-8")


class CopilotAgent(DekkAgent):
    """GitHub Copilot target generation."""

    target = TARGET_COPILOT

    def generate(self, context: AgentContext) -> list[str]:
        copilot_dir = context.project_root / COPILOT_DIR
        copilot_dir.mkdir(parents=True, exist_ok=True)
        copilot_path = copilot_dir / COPILOT_INSTRUCTIONS
        copilot_path.write_text(context.project_content, encoding="utf-8")
        generate_copilot_per_directory(context.project_root, context.rules)
        return [
            f"{COPILOT_DIR}/{COPILOT_INSTRUCTIONS}",
            f"{COPILOT_DIR}/{COPILOT_PER_DIR}/ ({len(context.rules)} rules)",
        ]

    def clean(self, context: AgentContext) -> list[str]:
        removed: list[str] = []
        removed.extend(
            remove_file(
                context.project_root / COPILOT_DIR / COPILOT_INSTRUCTIONS,
                f"{COPILOT_DIR}/{COPILOT_INSTRUCTIONS}",
            )
        )
        removed.extend(
            remove_tree(
                context.project_root / COPILOT_DIR / COPILOT_PER_DIR,
                f"{COPILOT_DIR}/{COPILOT_PER_DIR}/",
            )
        )
        remove_dir_if_empty(context.project_root / COPILOT_DIR)
        return removed


__all__ = ["CopilotAgent", "generate_copilot_per_directory"]



"""Tests for library path resolution module."""

from __future__ import annotations

import os
from pathlib import Path
from unittest.mock import patch

import pytest

from dekk.detection.detect import PlatformInfo
from dekk.detection.libpath import LibraryPathInfo, LibraryPathResolver


def _p(*parts: str) -> str:
    """Normalize a Unix-style path string for the current platform.

    On Unix this is a no-op; on Windows ``Path("/opt/lib")`` becomes
    ``\\opt\\lib``, which matches how the production code normalises via
    ``str(Path(...))``.
    """
    return str(Path(*parts))

# ---------------------------------------------------------------------------
# LibraryPathResolver construction
# ---------------------------------------------------------------------------


class TestLibraryPathResolverConstruction:
    def test_for_current_platform(self):
        resolver = LibraryPathResolver.for_current_platform()
        assert resolver.platform_info is not None
        assert resolver.env_var in ("LD_LIBRARY_PATH", "DYLD_LIBRARY_PATH", "PATH")

    def test_for_linux_platform(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        assert resolver.env_var == "LD_LIBRARY_PATH"
        assert resolver.platform_info.is_linux

    def test_for_macos_platform(self):
        resolver = LibraryPathResolver.for_platform("Darwin", arch="arm64")
        assert resolver.env_var == "DYLD_LIBRARY_PATH"
        assert resolver.platform_info.is_macos

    def test_for_windows_platform(self):
        resolver = LibraryPathResolver.for_platform("Windows")
        assert resolver.env_var == "PATH"
        assert resolver.platform_info.is_windows

    def test_explicit_platform_info(self):
        info = PlatformInfo(os="Linux", arch="aarch64")
        resolver = LibraryPathResolver(platform_info=info)
        assert resolver.env_var == "LD_LIBRARY_PATH"
        assert resolver.platform_info is info

    def test_unknown_os_defaults_to_ld_library_path(self):
        info = PlatformInfo(os="FreeBSD", arch="x86_64")
        resolver = LibraryPathResolver(platform_info=info)
        assert resolver.env_var == "LD_LIBRARY_PATH"


# ---------------------------------------------------------------------------
# Prepend / Append
# ---------------------------------------------------------------------------


class TestPrependAppend:
    def setup_method(self):
        self.resolver = LibraryPathResolver.for_platform("Linux")

    def test_prepend_single(self):
        self.resolver.prepend("/opt/lib")
        info = self.resolver.resolve()
        assert _p("/opt/lib") in info.paths

    def test_prepend_multiple(self):
        self.resolver.prepend("/opt/lib", "/usr/local/lib")
        info = self.resolver.resolve()
        assert info.paths[0] == _p("/opt/lib")
        assert info.paths[1] == _p("/usr/local/lib")

    def test_append_single(self):
        self.resolver.append("/fallback/lib")
        info = self.resolver.resolve()
        assert _p("/fallback/lib") in info.paths

    def test_prepend_before_append(self):
        self.resolver.prepend("/first")
        self.resolver.append("/last")
        info = self.resolver.resolve()
        first_idx = info.paths.index(_p("/first"))
        last_idx = info.paths.index(_p("/last"))
        assert first_idx < last_idx

    def test_chaining(self):
        info = self.resolver.prepend("/opt/lib").append("/fallback/lib").resolve()
        assert _p("/opt/lib") in info.paths
        assert _p("/fallback/lib") in info.paths

    def test_deduplication_across_prepend(self):
        self.resolver.prepend("/opt/lib")
        self.resolver.prepend("/opt/lib")
        info = self.resolver.resolve()
        assert info.paths.count(_p("/opt/lib")) == 1

    def test_deduplication_across_append(self):
        self.resolver.append("/opt/lib")
        self.resolver.append("/opt/lib")
        info = self.resolver.resolve()
        assert info.paths.count(_p("/opt/lib")) == 1

    def test_deduplication_prepend_vs_append(self):
        self.resolver.prepend("/opt/lib")
        self.resolver.append("/opt/lib")  # Should be skipped
        info = self.resolver.resolve()
        assert info.paths.count(_p("/opt/lib")) == 1

    def test_deduplication_append_vs_prepend(self):
        self.resolver.append("/opt/lib")
        self.resolver.prepend("/opt/lib")  # Should be skipped
        info = self.resolver.resolve()
        assert info.paths.count(_p("/opt/lib")) == 1


# ---------------------------------------------------------------------------
# Resolve with existing env
# ---------------------------------------------------------------------------


class TestResolveWithEnv:
    def test_merges_with_existing_env(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/new/lib")

        with patch.dict(os.environ, {"LD_LIBRARY_PATH": "/existing/lib"}):
            info = resolver.resolve()

        assert info.paths[0] == _p("/new/lib")
        assert _p("/existing/lib") in info.paths

    def test_existing_between_prepend_and_append(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/first")
        resolver.append("/last")

        with patch.dict(os.environ, {"LD_LIBRARY_PATH": "/middle"}):
            info = resolver.resolve()

        assert info.paths == (_p("/first"), _p("/middle"), _p("/last"))

    def test_deduplicates_against_existing(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/existing/lib")

        with patch.dict(os.environ, {"LD_LIBRARY_PATH": "/existing/lib:/other/lib"}):
            info = resolver.resolve()

        # /existing/lib should appear only once (from prepend, which takes priority)
        assert info.paths.count(_p("/existing/lib")) == 1
        assert info.paths[0] == _p("/existing/lib")  # prepend position wins

    def test_empty_env(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            info = resolver.resolve()

        assert info.paths == (_p("/opt/lib"),)

    def test_empty_everything(self):
        resolver = LibraryPathResolver.for_platform("Linux")

        with patch.dict(os.environ, {}, clear=True):
            info = resolver.resolve()

        assert info.paths == ()

    def test_macos_uses_dyld(self):
        resolver = LibraryPathResolver.for_platform("Darwin")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {"DYLD_LIBRARY_PATH": "/existing"}, clear=True):
            info = resolver.resolve()

        assert info.env_var == "DYLD_LIBRARY_PATH"
        assert _p("/opt/lib") in info.paths
        assert _p("/existing") in info.paths


# ---------------------------------------------------------------------------
# LibraryPathInfo
# ---------------------------------------------------------------------------


class TestLibraryPathInfo:
    def test_as_string_linux(self):
        info = LibraryPathInfo(
            env_var="LD_LIBRARY_PATH",
            paths=("/opt/lib", "/usr/local/lib"),
            platform=PlatformInfo(os="Linux", arch="x86_64"),
        )
        assert info.as_string == "/opt/lib:/usr/local/lib"

    def test_as_string_windows(self):
        info = LibraryPathInfo(
            env_var="PATH",
            paths=("C:\\lib", "C:\\usr\\lib"),
            platform=PlatformInfo(os="Windows", arch="x86_64"),
        )
        assert info.as_string == "C:\\lib;C:\\usr\\lib"

    def test_contains(self):
        info = LibraryPathInfo(
            env_var="LD_LIBRARY_PATH",
            paths=("/opt/lib", "/usr/local/lib"),
            platform=PlatformInfo(os="Linux", arch="x86_64"),
        )
        assert info.contains("/opt/lib")
        assert not info.contains("/missing/lib")

    def test_contains_normalizes_path(self):
        info = LibraryPathInfo(
            env_var="LD_LIBRARY_PATH",
            paths=("/opt/lib",),
            platform=PlatformInfo(os="Linux", arch="x86_64"),
        )
        # Trailing slash should still match
        assert info.contains("/opt/lib/")

    def test_empty_paths(self):
        info = LibraryPathInfo(
            env_var="LD_LIBRARY_PATH",
            paths=(),
            platform=PlatformInfo(os="Linux", arch="x86_64"),
        )
        assert info.as_string == ""
        assert not info.contains("/any")


# ---------------------------------------------------------------------------
# Apply
# ---------------------------------------------------------------------------


class TestApply:
    def test_apply_sets_env_var(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            info = resolver.apply()
            assert os.environ["LD_LIBRARY_PATH"] == _p("/opt/lib")
            assert info.env_var == "LD_LIBRARY_PATH"

    def test_apply_preserves_existing(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/new/lib")

        with patch.dict(os.environ, {"LD_LIBRARY_PATH": "/existing/lib"}):
            resolver.apply()
            assert os.environ["LD_LIBRARY_PATH"] == _p("/new/lib") + ":" + _p("/existing/lib")

    def test_apply_empty_does_not_set(self):
        resolver = LibraryPathResolver.for_platform("Linux")

        with patch.dict(os.environ, {}, clear=True):
            resolver.apply()
            assert "LD_LIBRARY_PATH" not in os.environ


# ---------------------------------------------------------------------------
# to_env_var integration
# ---------------------------------------------------------------------------


class TestToEnvVar:
    def test_to_env_var(self):
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            name, value = resolver.to_env_var()

        assert name == "LD_LIBRARY_PATH"
        assert value == _p("/opt/lib")

    def test_to_env_var_macos(self):
        resolver = LibraryPathResolver.for_platform("Darwin")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            name, value = resolver.to_env_var()

        assert name == "DYLD_LIBRARY_PATH"
        assert value == _p("/opt/lib")

    def test_works_with_env_var_dataclass(self):
        """Verify the tuple integrates with dekk.shell.EnvVar."""
        from dekk.shell import EnvVar

        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            name, value = resolver.to_env_var()

        env = EnvVar(name=name, value=value, prepend_path=True)
        assert env.name == "LD_LIBRARY_PATH"
        assert env.value == _p("/opt/lib")


# ---------------------------------------------------------------------------
# APXM integration scenario
# ---------------------------------------------------------------------------


class TestApxmIntegration:
    """End-to-end test showing how APXM would use LibraryPathResolver."""

    def test_apxm_conda_lib_setup(self):
        """Simulate setting up MLIR/LLVM library paths for APXM."""
        prefix = "/home/user/miniforge3/envs/apxm"
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend(
            f"{prefix}/lib",
            f"{prefix}/lib/cmake/mlir",
        )

        with patch.dict(os.environ, {}, clear=True):
            info = resolver.resolve()

        assert info.env_var == "LD_LIBRARY_PATH"
        assert _p(f"{prefix}/lib") in info.paths
        assert _p(f"{prefix}/lib/cmake/mlir") in info.paths
        assert info.paths[0] == _p(f"{prefix}/lib")

    def test_apxm_macos_cross_platform(self):
        """Same setup but targeting macOS."""
        prefix = "/opt/homebrew/opt/llvm"
        resolver = LibraryPathResolver.for_platform("Darwin", arch="arm64")
        resolver.prepend(f"{prefix}/lib")

        with patch.dict(os.environ, {}, clear=True):
            info = resolver.resolve()

        assert info.env_var == "DYLD_LIBRARY_PATH"
        assert _p(f"{prefix}/lib") in info.paths

    def test_linux_current_platform(self):
        """Verify for_current_platform matches the actual runtime platform."""
        resolver = LibraryPathResolver.for_current_platform()

        expected_var = {
            "Linux": "LD_LIBRARY_PATH",
            "Darwin": "DYLD_LIBRARY_PATH",
            "Windows": "PATH",
        }.get(resolver.platform_info.os, "LD_LIBRARY_PATH")

        assert resolver.env_var == expected_var


# ---------------------------------------------------------------------------
# configure_builder integration
# ---------------------------------------------------------------------------


class TestConfigureBuilder:
    """Test LibraryPathResolver.configure_builder with both builder types."""

    def test_configure_env_builder(self):
        """configure_builder with dekk.execution.env.EnvVarBuilder (has set_from_path)."""
        from dekk.execution.env import EnvVarBuilder

        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib", "/usr/local/lib")

        builder = EnvVarBuilder()
        with patch.dict(os.environ, {}, clear=True):
            resolver.configure_builder(builder)

        snap = builder.build()
        value = snap.get("LD_LIBRARY_PATH")
        assert value is not None
        assert _p("/opt/lib") in value
        assert _p("/usr/local/lib") in value

    def test_configure_toolchain_builder(self):
        """configure_builder with dekk.execution.toolchain.EnvVarBuilder (has prepend_var)."""
        from dekk.execution.toolchain import EnvVarBuilder as TcBuilder

        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        builder = TcBuilder()
        with patch.dict(os.environ, {}, clear=True):
            resolver.configure_builder(builder)

        config = builder.build()
        ld_vars = [v for v in config.env_vars if v.name == "LD_LIBRARY_PATH"]
        assert len(ld_vars) == 1
        assert ld_vars[0].prepend_path is True

    def test_configure_unsupported_builder(self):
        """configure_builder raises TypeError for unknown builder types."""
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/opt/lib")

        with patch.dict(os.environ, {}, clear=True):
            with pytest.raises(TypeError, match="Unsupported builder"):
                resolver.configure_builder(object())

    def test_configure_builder_empty_resolves_noop(self):
        """configure_builder with no paths does nothing."""
        from dekk.execution.env import EnvVarBuilder

        resolver = LibraryPathResolver.for_platform("Linux")
        builder = EnvVarBuilder()

        with patch.dict(os.environ, {}, clear=True):
            resolver.configure_builder(builder)

        snap = builder.build()
        assert snap.get("LD_LIBRARY_PATH") is None

    def test_configure_macos_builder(self):
        """configure_builder sets DYLD_LIBRARY_PATH for macOS."""
        from dekk.execution.env import EnvVarBuilder

        resolver = LibraryPathResolver.for_platform("Darwin")
        resolver.prepend("/opt/lib")

        builder = EnvVarBuilder()
        with patch.dict(os.environ, {}, clear=True):
            resolver.configure_builder(builder)

        snap = builder.build()
        assert snap.get("DYLD_LIBRARY_PATH") is not None
        assert snap.get("LD_LIBRARY_PATH") is None


# ---------------------------------------------------------------------------
# Cross-module: libpath + toolchain end-to-end
# ---------------------------------------------------------------------------


class TestLibpathToolchainIntegration:
    """End-to-end tests combining libpath with toolchain profiles."""

    def test_libpath_then_cmake_toolchain(self):
        """LibraryPathResolver and CMakeToolchain can both contribute to the same builder."""
        from dekk.execution.toolchain import CMakeToolchain
        from dekk.execution.toolchain import EnvVarBuilder as TcBuilder

        prefix = "/opt/conda/envs/apxm"
        resolver = LibraryPathResolver.for_platform("Linux")
        resolver.prepend("/custom/runtime/lib")

        builder = TcBuilder(app_name="apxm")

        with patch.dict(os.environ, {}, clear=True):
            # First, libpath resolver contributes
            resolver.configure_builder(builder)
            # Then, CMakeToolchain contributes
            from unittest.mock import patch as mpatch

            with mpatch("dekk.execution.os.posix.platform.system", return_value="Linux"):
                cmake = CMakeToolchain(prefix=Path(prefix))
                cmake.configure(builder)

        config = builder.build()
        ld_vars = [v for v in config.env_vars if v.name == "LD_LIBRARY_PATH"]
        # Both the resolver's path and cmake's prefix/lib should be present
        ld_values = [v.value for v in ld_vars]
        assert _p("/custom/runtime/lib") in ld_values
        assert _p(prefix, "lib") in ld_values



"""Self-contained wrapper script generation for zero-activation CLI tools.

The WrapperGenerator creates shell scripts that bake in all environment
variables, PATH entries, and conda configuration from a .dekk.toml spec.
The result is a single executable file that activates the full project
environment and execs the target binary -- no manual activation needed.

Usage:
    from dekk.execution.wrapper import WrapperGenerator

    # From .dekk.toml (simplest)
    result = WrapperGenerator.install_from_spec(
        spec_file=Path(".dekk.toml"),
        target=Path("bin/myapp"),
        name="myapp",
    )

    # With a Python entry point
    result = WrapperGenerator.install_from_spec(
        spec_file=Path(".dekk.toml"),
        target=Path("tools/cli.py"),
        python=Path("/path/to/conda/bin/python3"),
        name="myapp",
    )
"""

from __future__ import annotations

import os
from pathlib import Path

from ..cli.errors import NotFoundError, ValidationError
from ..environment.activation import ActivationResult, EnvironmentActivator
from ..environment.spec import EnvironmentSpec
from .install import DEFAULT_INSTALL_DIRNAME, InstallResult
from .os import get_dekk_os

# ---------------------------------------------------------------------------
# WrapperGenerator
# ---------------------------------------------------------------------------


class WrapperGenerator:
    """Generate and install self-contained wrapper scripts.

    A wrapper is a tiny script that:

    1. Exports every environment variable needed by the project (hardcoded
       absolute values -- no runtime detection).
    2. Prepends the required directories to ``PATH``.
    3. Launches the target binary (or ``python target``) with the fully
       prepared environment.

    On POSIX systems the wrapper is a ``#!/bin/sh`` script. On Windows it is
    a ``.cmd`` launcher so it can be invoked directly from both Command Prompt
    and PowerShell without relying on execution-policy changes.

    The class is intentionally stateless: every method is either a
    ``@staticmethod`` or a ``@classmethod`` that composes the statics.
    """

    # ------------------------------------------------------------------ #
    # Core generation
    # ------------------------------------------------------------------ #

    @staticmethod
    def generate(
        target: Path,
        env_vars: dict[str, str],
        path_prepends: list[str],
        project_name: str,
        *,
        prepend_vars: dict[str, str] | None = None,
        python: Path | None = None,
    ) -> str:
        """Generate a self-contained wrapper script for the current platform.

        Args:
            target: Absolute path to the binary (or Python script) to run.
            env_vars: Mapping of environment-variable names to their values.
                Values are embedded literally (hard set).
            path_prepends: Directories to prepend to ``$PATH`` (in order).
            project_name: Human-readable project label (used in the header
                comment only).
            prepend_vars: Mapping of variable names to values that should be
                *prepended* to existing values (e.g. ``LD_LIBRARY_PATH``).
                Each is rendered as ``export VAR="value:$VAR"``.
            python: If provided, the wrapper will ``exec python target "$@"``
                instead of ``exec target "$@"``.

        Returns:
            The complete wrapper script as a string, ready to be written to
            a file and made executable or invoked directly.

        Raises:
            NotFoundError: If *target* does not exist on disk.
        """
        target = target.resolve()
        if not target.exists():
            raise NotFoundError(
                f"Target does not exist: {target}",
                hint="Build or install the target binary first",
            )

        if python is not None:
            python = python.resolve()
            if not python.exists():
                raise NotFoundError(
                    f"Python interpreter does not exist: {python}",
                    hint="Check the conda environment or virtualenv path",
                )

        dekk_os = get_dekk_os()
        return dekk_os.render_wrapper(
            target=target,
            env_vars=env_vars,
            path_prepends=path_prepends,
            project_name=project_name,
            prepend_vars=prepend_vars,
            python=python,
        )

    # ------------------------------------------------------------------ #
    # Construction helpers
    # ------------------------------------------------------------------ #

    @classmethod
    def from_spec(
        cls,
        spec_file: Path | EnvironmentSpec,
        project_root: Path,
        target: Path,
        *,
        python: Path | None = None,
    ) -> str:
        """Generate a wrapper from a ``.dekk.toml`` spec (or ``EnvironmentSpec``).

        This resolves the full environment (conda prefix, env vars, PATH
        entries) via :class:`~dekk.environment.activation.EnvironmentActivator` and
        then delegates to :meth:`generate`.

        Args:
            spec_file: Path to a ``.dekk.toml`` file, or an already-parsed
                :class:`~dekk.environment.spec.EnvironmentSpec`.
            project_root: Root directory of the project (used for placeholder
                expansion and as the activation context).
            target: Absolute (or project-relative) path to the binary.
            python: Optional Python interpreter for script targets.

        Returns:
            The wrapper script as a string.
        """
        if isinstance(spec_file, EnvironmentSpec):
            spec = spec_file
        else:
            spec_file = Path(spec_file).resolve()
            spec = EnvironmentSpec.from_file(spec_file)

        project_root = Path(project_root).resolve()

        activator = EnvironmentActivator(spec, project_root)
        result = activator.activate()

        return cls._generate_from_activation(
            result,
            target=target,
            project_name=spec.project_name,
            python=python,
        )

    @classmethod
    def from_activation(
        cls,
        activation: ActivationResult,
        target: Path,
        project_name: str,
        *,
        python: Path | None = None,
    ) -> str:
        """Generate a wrapper from an existing :class:`ActivationResult`.

        Useful when the caller has already performed activation and wants
        to reuse the result.

        Args:
            activation: A previously obtained activation result.
            target: Absolute path to the binary.
            project_name: Project label for the header comment.
            python: Optional Python interpreter for script targets.

        Returns:
            The wrapper script as a string.
        """
        return cls._generate_from_activation(
            activation,
            target=target,
            project_name=project_name,
            python=python,
        )

    # ------------------------------------------------------------------ #
    # Installation
    # ------------------------------------------------------------------ #

    @staticmethod
    def install(
        script: str,
        name: str,
        *,
        install_dir: Path | None = None,
    ) -> InstallResult:
        """Write a wrapper script to disk and make it executable.

        Args:
            script: The wrapper script content (as returned by
                :meth:`generate`, :meth:`from_spec`, etc.).
            name: File name for the wrapper (e.g. ``"myapp"``).  Must not
                contain path separators.
            install_dir: Directory in which to place the wrapper. Defaults to
                ``.install`` in the current working directory.

        Returns:
            An :class:`~dekk.execution.install.InstallResult` with the written path,
            whether the directory is in ``$PATH``, and a human-readable
            message.

        Raises:
            ValidationError: If *name* contains a path separator.
        """
        if os.sep in name or (os.altsep and os.altsep in name):
            raise ValidationError(
                f"Wrapper name must not contain path separators: {name!r}",
                hint="Pass just the file name, e.g. 'myapp'",
            )

        dekk_os = get_dekk_os()
        wrapper_name = dekk_os.wrapper_filename(name)

        if install_dir is None:
            install_dir = _default_install_dir()

        install_dir = Path(install_dir).resolve()
        install_dir.mkdir(parents=True, exist_ok=True)

        wrapper_path = install_dir / wrapper_name
        wrapper_path.write_text(script, encoding="utf-8")
        dekk_os.make_wrapper_executable(wrapper_path)

        # Check if install_dir is in $PATH
        in_path = _dir_in_path(install_dir)

        if in_path:
            message = f"Installed wrapper '{wrapper_name}' -> {wrapper_path}"
        else:
            message = (
                f"Installed wrapper '{wrapper_name}' -> {wrapper_path} "
                f"(add {install_dir} to PATH to use it directly)"
            )

        return InstallResult(bin_path=wrapper_path, in_path=in_path, message=message)

    @staticmethod
    def uninstall(
        name: str,
        *,
        install_dir: Path | None = None,
    ) -> InstallResult:
        """Remove a previously installed wrapper script.

        Idempotent: returns success even if the wrapper does not exist.

        Args:
            name: Wrapper file name (e.g. ``"myapp"``).
            install_dir: Directory the wrapper lives in. Defaults to
                ``.install`` in the current working directory.

        Returns:
            :class:`~dekk.execution.install.InstallResult` describing what was removed.
        """
        dekk_os = get_dekk_os()
        wrapper_name = dekk_os.wrapper_filename(name)

        if install_dir is None:
            install_dir = _default_install_dir()

        install_dir = Path(install_dir).resolve()
        wrapper_path = install_dir / wrapper_name

        if wrapper_path.exists():
            wrapper_path.unlink()
            message = f"Removed wrapper '{wrapper_name}' from {install_dir}"
        else:
            message = f"Wrapper '{wrapper_name}' not found in {install_dir} (nothing to remove)"

        return InstallResult(
            bin_path=wrapper_path, in_path=_dir_in_path(install_dir), message=message
        )

    # ------------------------------------------------------------------ #
    # Convenience: from_spec + install in one shot
    # ------------------------------------------------------------------ #

    @classmethod
    def install_from_spec(
        cls,
        spec_file: Path | EnvironmentSpec,
        target: Path,
        name: str,
        *,
        python: Path | None = None,
        install_dir: Path | None = None,
        project_root: Path | None = None,
    ) -> InstallResult:
        """Generate a wrapper from a spec and install it in one step.

        This is the highest-level entry point: pass a ``.dekk.toml`` path,
        the target binary, and a wrapper name, and get back an installed,
        executable wrapper.

        Args:
            spec_file: Path to ``.dekk.toml`` (or an ``EnvironmentSpec``).
            target: Path to the binary or Python script to wrap.
            name: Wrapper file name (e.g. ``"myapp"``).
            python: Optional Python interpreter.
            install_dir: Where to install (default: the project's ``.install`` directory).
            project_root: Project root directory.  If *None*, inferred from
                *spec_file*'s parent directory (when a path is given) or the
                current working directory (when an ``EnvironmentSpec`` is
                given).

        Returns:
            :class:`~dekk.execution.install.InstallResult`.
        """
        if project_root is not None:
            root = Path(project_root).resolve()
        elif isinstance(spec_file, Path):
            root = Path(spec_file).resolve().parent
        else:
            root = Path.cwd().resolve()

        script = cls.from_spec(
            spec_file,
            project_root=root,
            target=target,
            python=python,
        )

        return cls.install(
            script, name, install_dir=install_dir or (root / DEFAULT_INSTALL_DIRNAME)
        )

    # ------------------------------------------------------------------ #
    # Internal helpers
    # ------------------------------------------------------------------ #

    @classmethod
    def _generate_from_activation(
        cls,
        activation: ActivationResult,
        target: Path,
        project_name: str,
        *,
        python: Path | None = None,
    ) -> str:
        """Translate an ``ActivationResult`` into ``generate()`` arguments.

        The ``ActivationResult.env_vars`` dict may contain a ``"PATH"`` key
        whose value holds the directories to prepend.  We split that out
        into the ``path_prepends`` list so that :meth:`generate` can build
        the correct ``export PATH="...:$PATH"`` line.
        """
        env_vars = dict(activation.env_vars)
        path_prepends: list[str] = []
        prepend_vars: dict[str, str] = {}

        # Variables that should be prepended to existing values, not hard-set.
        _PREPEND_KEYS = {
            "PATH",
            "LD_LIBRARY_PATH",
            "DYLD_LIBRARY_PATH",
            "PYTHONPATH",
            "PKG_CONFIG_PATH",
        }

        # Extract PATH entries from the env dict (the activator merges them
        # into a single colon-separated value under the "PATH" key).
        # Filter out entries that are already in the current $PATH to keep
        # the wrapper lean — it appends $PATH at the end anyway.
        raw_path = env_vars.pop("PATH", "")
        if raw_path:
            current_path = set(os.environ.get("PATH", "").split(os.pathsep))
            seen: set[str] = set()
            for p in raw_path.split(os.pathsep):
                if p and p not in current_path and p not in seen:
                    path_prepends.append(p)
                    seen.add(p)

        # Extract other prepend-style vars (LD_LIBRARY_PATH, etc.)
        for key in list(env_vars):
            if key in _PREPEND_KEYS:
                prepend_vars[key] = env_vars.pop(key)

        return cls.generate(
            target=target,
            env_vars=env_vars,
            path_prepends=path_prepends,
            project_name=project_name,
            prepend_vars=prepend_vars,
            python=python,
        )


# ---------------------------------------------------------------------------
# Module-private helpers
# ---------------------------------------------------------------------------


def _default_install_dir(project_root: Path | None = None) -> Path:
    """Return the default project-local install directory."""
    root = project_root.resolve() if project_root is not None else Path.cwd().resolve()
    return root / DEFAULT_INSTALL_DIRNAME


def _dir_in_path(directory: Path) -> bool:
    """Return ``True`` if *directory* is present in the current ``$PATH``."""
    dir_resolved = os.path.normcase(str(directory.resolve()))
    for entry in os.environ.get("PATH", "").split(os.pathsep):
        if not entry:
            continue
        try:
            if os.path.normcase(str(Path(entry).resolve())) == dir_resolved:
                return True
        except (OSError, ValueError):
            continue
    return False



"""Shared abstractions for agent-specific generation targets."""

from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path

from dekk.agents.discovery import RuleDefinition, SkillDefinition


@dataclass(frozen=True)
class AgentContext:
    """Shared generation context for a concrete agent target."""

    project_root: Path
    source_dir: Path
    source_dir_name: str
    project_name: str
    cli_name: str | None
    project_content: str
    skills: list[SkillDefinition]
    rules: list[RuleDefinition]


class DekkAgent(ABC):
    """Contract implemented by each agent-specific target generator."""

    target: str

    @abstractmethod
    def generate(self, context: AgentContext) -> list[str]:
        """Generate target-specific files and return user-facing output labels."""

    def clean(self, context: AgentContext) -> list[str]:
        """Remove target-specific generated files and return removed labels."""
        return []


__all__ = ["AgentContext", "DekkAgent"]



"""Tests for toolchain profiles and EnvVarBuilder."""

from __future__ import annotations

import os
import sys
from pathlib import Path
from unittest.mock import patch

import pytest

from dekk.execution.os import PosixDekkOS, WindowsDekkOS
from dekk.execution.toolchain import (
    CMakeToolchain,
    CondaToolchain,
    EnvVarBuilder,
    ToolchainProfile,
)
from dekk.shell import ActivationConfig, ActivationScriptBuilder, EnvVar, ShellKind

# ---------------------------------------------------------------------------
# EnvVarBuilder
# ---------------------------------------------------------------------------


class TestEnvVarBuilder:
    def test_empty_builder(self):
        builder = EnvVarBuilder()
        config = builder.build()
        assert config.env_vars == ()
        assert config.path_prepends == ()
        assert config.app_name == ""
        assert config.banner is None

    def test_app_name(self):
        builder = EnvVarBuilder(app_name="apxm")
        config = builder.build()
        assert config.app_name == "apxm"

    def test_set_var(self):
        builder = EnvVarBuilder()
        builder.set_var("FOO", "/some/path")
        config = builder.build()
        assert len(config.env_vars) == 1
        assert config.env_vars[0] == EnvVar(name="FOO", value="/some/path")
        assert config.env_vars[0].prepend_path is False

    def test_prepend_var(self):
        builder = EnvVarBuilder()
        builder.prepend_var("LD_LIBRARY_PATH", "/opt/lib")
        config = builder.build()
        assert len(config.env_vars) == 1
        assert config.env_vars[0] == EnvVar(
            name="LD_LIBRARY_PATH", value="/opt/lib", prepend_path=True
        )

    def test_prepend_path(self):
        builder = EnvVarBuilder()
        builder.prepend_path("/opt/conda/bin")
        builder.prepend_path(Path("/usr/local/bin"))
        config = builder.build()
        assert config.path_prepends == ("/opt/conda/bin", str(Path("/usr/local/bin")))

    def test_set_banner(self):
        builder = EnvVarBuilder()
        builder.set_banner("Toolchain activated")
        config = builder.build()
        assert config.banner == "Toolchain activated"

    def test_build_produces_frozen_config(self):
        builder = EnvVarBuilder(app_name="test")
        builder.set_var("X", "1")
        config = builder.build()
        assert isinstance(config, ActivationConfig)
        with pytest.raises(AttributeError):
            config.app_name = "changed"  # type: ignore[misc]

    def test_to_env_dict(self):
        builder = EnvVarBuilder()
        builder.set_var("MLIR_DIR", "/opt/mlir")
        builder.set_var("LLVM_DIR", "/opt/llvm")
        builder.prepend_path("/opt/bin")
        d = builder.to_env_dict()
        assert d["MLIR_DIR"] == "/opt/mlir"
        assert d["LLVM_DIR"] == "/opt/llvm"
        assert d["PATH"] == "/opt/bin"

    def test_to_env_dict_multiple_paths(self):
        builder = EnvVarBuilder()
        builder.prepend_path("/a")
        builder.prepend_path("/b")
        d = builder.to_env_dict()
        assert d["PATH"] == os.pathsep.join(("/a", "/b"))

    def test_to_env_dict_uses_os_pathsep(self, monkeypatch: pytest.MonkeyPatch):
        builder = EnvVarBuilder()
        builder.prepend_var("PYTHONPATH", "C:/first")
        builder.prepend_var("PYTHONPATH", "C:/second")
        builder.prepend_path("C:/bin")
        monkeypatch.setattr("dekk.execution.toolchain.builder.os.pathsep", ";")
        d = builder.to_env_dict()
        assert d["PYTHONPATH"] == "C:/first;C:/second"
        assert d["PATH"] == "C:/bin"

    def test_to_env_dict_no_path(self):
        builder = EnvVarBuilder()
        builder.set_var("X", "1")
        d = builder.to_env_dict()
        assert "PATH" not in d

    def test_multiple_profiles(self):
        """Builder accumulates vars from multiple configure calls."""
        builder = EnvVarBuilder(app_name="test")
        builder.set_var("A", "1")
        builder.set_var("B", "2")
        builder.prepend_path("/first")
        builder.set_var("C", "3")
        builder.prepend_path("/second")
        config = builder.build()
        assert len(config.env_vars) == 3
        assert config.path_prepends == ("/first", "/second")


# ---------------------------------------------------------------------------
# CMakeToolchain
# ---------------------------------------------------------------------------


class TestCMakeToolchain:
    def setup_method(self):
        self.prefix = Path("/opt/conda/envs/apxm")
        self.tc = CMakeToolchain(prefix=self.prefix)

    def test_frozen(self):
        with pytest.raises(AttributeError):
            self.tc.prefix = Path("/other")  # type: ignore[misc]

    @pytest.mark.skipif(sys.platform == "win32", reason="Tests Unix conda layout")
    def test_properties(self):
        assert self.tc.mlir_dir == self.prefix / "lib" / "cmake" / "mlir"
        assert self.tc.llvm_dir == self.prefix / "lib" / "cmake" / "llvm"
        assert self.tc.lib_dir == self.prefix / "lib"
        assert self.tc.bin_dir == self.prefix / "bin"

    @patch("dekk.execution.toolchain.cmake.get_dekk_os", return_value=WindowsDekkOS())
    def test_properties_windows(self, _mock):
        prefix = Path("C:/miniforge/envs/apxm")
        tc = CMakeToolchain(prefix=prefix)
        assert tc.mlir_dir == prefix / "Library" / "lib" / "cmake" / "mlir"
        assert tc.llvm_dir == prefix / "Library" / "lib" / "cmake" / "llvm"
        assert tc.lib_dir == prefix / "Library" / "lib"
        assert tc.bin_dirs == (
            prefix,
            prefix / "Library" / "mingw-w64" / "bin",
            prefix / "Library" / "usr" / "bin",
            prefix / "Library" / "bin",
            prefix / "Scripts",
            prefix / "bin",
        )

    @patch("dekk.execution.os.posix.platform.system", return_value="Linux")
    @patch("dekk.execution.toolchain.cmake.get_dekk_os", return_value=PosixDekkOS())
    def test_configure_linux(self, _os_mock, _platform_mock):
        builder = EnvVarBuilder(app_name="apxm")
        self.tc.configure(builder)
        config = builder.build()

        var_names = [v.name for v in config.env_vars]
        assert "CMAKE_PREFIX_PATH" in var_names
        assert "LD_LIBRARY_PATH" in var_names
        assert "DYLD_LIBRARY_PATH" not in var_names

        cmake_prefix_var = next(v for v in config.env_vars if v.name == "CMAKE_PREFIX_PATH")
        assert cmake_prefix_var.value == str(self.prefix)
        assert cmake_prefix_var.prepend_path is True

        ld_var = next(v for v in config.env_vars if v.name == "LD_LIBRARY_PATH")
        assert ld_var.value == str(self.prefix / "lib")
        assert ld_var.prepend_path is True

        assert str(self.prefix / "bin") in config.path_prepends

    @patch("dekk.execution.os.posix.platform.system", return_value="Darwin")
    @patch("dekk.execution.toolchain.cmake.get_dekk_os", return_value=PosixDekkOS())
    def test_configure_macos(self, _os_mock, _platform_mock):
        builder = EnvVarBuilder()
        self.tc.configure(builder)
        config = builder.build()

        var_names = [v.name for v in config.env_vars]
        assert "DYLD_LIBRARY_PATH" in var_names
        assert "LD_LIBRARY_PATH" not in var_names

    @patch("dekk.execution.toolchain.cmake.get_dekk_os", return_value=WindowsDekkOS())
    def test_configure_windows(self, _mock):
        prefix = Path("C:/miniforge/envs/apxm")
        tc = CMakeToolchain(prefix=prefix)
        builder = EnvVarBuilder()
        tc.configure(builder)
        config = builder.build()

        var_names = [v.name for v in config.env_vars]
        assert "CMAKE_PREFIX_PATH" in var_names
        assert "LD_LIBRARY_PATH" not in var_names
        assert "DYLD_LIBRARY_PATH" not in var_names
        assert str(prefix) in config.path_prepends
        assert str(prefix / "Library" / "mingw-w64" / "bin") in config.path_prepends
        assert str(prefix / "Library" / "usr" / "bin") in config.path_prepends
        assert str(prefix / "Library" / "bin") in config.path_prepends
        assert str(prefix / "Scripts") in config.path_prepends
        assert str(prefix / "bin") in config.path_prepends

    @patch("dekk.execution.os.posix.platform.system", return_value="Linux")
    @patch("dekk.execution.toolchain.cmake.get_dekk_os", return_value=PosixDekkOS())
    def test_extra_lib_dirs(self, _os_mock, _platform_mock):
        tc = CMakeToolchain(
            prefix=self.prefix,
            extra_lib_dirs=("/opt/release/lib", "/opt/release"),
        )
        builder = EnvVarBuilder()
        tc.configure(builder)
        config = builder.build()

        ld_vars = [v for v in config.env_vars if v.name == "LD_LIBRARY_PATH"]
        # extra_lib_dirs come first, then the prefix lib
        assert len(ld_vars) == 3
        assert ld_vars[0].value == "/opt/release/lib"
        assert ld_vars[1].value == "/opt/release"
        assert ld_vars[2].value == str(self.prefix / "lib")

    def test_generates_valid_activation_script(self):
        """End-to-end: CMakeToolchain -> EnvVarBuilder -> ActivationScriptBuilder."""
        builder = EnvVarBuilder(app_name="apxm")
        self.tc.configure(builder)
        config = builder.build()

        script = ActivationScriptBuilder().build(config, ShellKind.BASH)
        assert "export CMAKE_PREFIX_PATH=" in script
        assert str(self.prefix / "bin") in script


# ---------------------------------------------------------------------------
# CondaToolchain
# ---------------------------------------------------------------------------


class TestCondaToolchain:
    def setup_method(self):
        self.prefix = Path("/home/user/miniforge3/envs/apxm")
        self.tc = CondaToolchain(prefix=self.prefix, env_name="apxm")

    def test_frozen(self):
        with pytest.raises(AttributeError):
            self.tc.prefix = Path("/other")  # type: ignore[misc]

    def test_configure(self):
        builder = EnvVarBuilder()
        self.tc.configure(builder)
        config = builder.build()

        var_names = [v.name for v in config.env_vars]
        assert "CONDA_PREFIX" in var_names
        assert "CONDA_DEFAULT_ENV" in var_names

        prefix_var = next(v for v in config.env_vars if v.name == "CONDA_PREFIX")
        assert prefix_var.value == str(self.prefix)

        env_var = next(v for v in config.env_vars if v.name == "CONDA_DEFAULT_ENV")
        assert env_var.value == "apxm"

        assert str(self.prefix / "bin") in config.path_prepends

    def test_configure_without_env_name(self):
        tc = CondaToolchain(prefix=self.prefix)
        builder = EnvVarBuilder()
        tc.configure(builder)
        config = builder.build()

        var_names = [v.name for v in config.env_vars]
        assert "CONDA_PREFIX" in var_names
        assert "CONDA_DEFAULT_ENV" not in var_names

    @patch("dekk.execution.toolchain.conda.get_dekk_os", return_value=WindowsDekkOS())
    def test_configure_windows(self, _mock):
        prefix = Path("C:/miniforge/envs/apxm")
        tc = CondaToolchain(prefix=prefix, env_name="apxm")
        builder = EnvVarBuilder()
        tc.configure(builder)
        config = builder.build()

        assert str(prefix) in config.path_prepends
        assert str(prefix / "Library" / "mingw-w64" / "bin") in config.path_prepends
        assert str(prefix / "Library" / "usr" / "bin") in config.path_prepends
        assert str(prefix / "Library" / "bin") in config.path_prepends
        assert str(prefix / "Scripts") in config.path_prepends
        assert str(prefix / "bin") in config.path_prepends


# ---------------------------------------------------------------------------
# ToolchainProfile protocol
# ---------------------------------------------------------------------------


class TestToolchainProfileProtocol:
    def test_cmake_is_toolchain_profile(self):
        tc = CMakeToolchain(prefix=Path("/opt"))
        assert isinstance(tc, ToolchainProfile)

    def test_conda_is_toolchain_profile(self):
        tc = CondaToolchain(prefix=Path("/opt"))
        assert isinstance(tc, ToolchainProfile)

    def test_custom_toolchain_profile(self):
        """Any class with configure(builder) satisfies the protocol."""

        class CustomToolchain:
            def configure(self, builder: EnvVarBuilder) -> None:
                builder.set_var("CUSTOM", "yes")

        tc = CustomToolchain()
        assert isinstance(tc, ToolchainProfile)

        builder = EnvVarBuilder()
        tc.configure(builder)
        config = builder.build()
        assert config.env_vars[0].name == "CUSTOM"



"""
dekk - One config. Zero activation. Any project.

Detect your project's environment, activate it from a .dekk.toml spec,
and generate self-contained wrapper binaries that just work.

All public symbols are lazily loaded on first access via PEP 562 __getattr__.
This means ``import dekk`` is near-instant (<5ms) regardless of which
optional tracking integrations are installed.
"""

from __future__ import annotations

from importlib.metadata import PackageNotFoundError, version
from typing import Any

try:
    __version__ = version("dekk")
except PackageNotFoundError:
    __version__ = "1.6.4"

# ---------------------------------------------------------------------------
# Lazy import registry: module_path -> list of exported names
# ---------------------------------------------------------------------------

_MODULE_ATTRS: dict[str, list[str]] = {
    # -- Automatic Environment Setup --
    "dekk.environment.spec": [
        "ComponentSpec",
        "EnvironmentSpec",
        "InstallSpec",
        "ToolSpec",
        "CommandSpec",
        "NpmSpec",
        "AgentsSpec",
        "PythonSpec",
        "RuntimeEnvironmentSpec",
        "WrapSpec",
        "find_envspec",
    ],
    "dekk.environment.types": ["EnvironmentKind"],
    "dekk.environment.providers": ["DekkEnv", "DekkEnvSetupResult", "CondaEnv"],
    # -- Project Setup --
    "dekk.environment.setup": ["SetupResult", "run_setup"],
    # -- Install Pipeline --
    "dekk.environment.install": ["run_install"],
    "dekk.cli.install_runner": [
        "InstallRunner",
        "InstallRunnerResult",
        "StepResult",
        "select_components",
    ],
    # -- Agent Config Management --
    "dekk.agents": [
        "DekkAgent",
        "AgentConfigManager",
        "RuleDefinition",
        "SkillDefinition",
        "create_agents_app",
        "discover_rules",
        "discover_skills",
        "install_codex_skills",
        "parse_frontmatter",
        "scaffold_agents_dir",
    ],
    "dekk.environment.activation": ["EnvironmentActivator", "ActivationResult"],
    "dekk.execution.install": ["BinaryInstaller", "InstallResult"],
    "dekk.execution.wrapper": ["WrapperGenerator"],
    # -- Detection & Platform --
    "dekk.detection.detect": ["PlatformDetector", "PlatformInfo"],
    "dekk.detection.deps": [
        "DependencyChecker",
        "DependencySpec",
        "DependencyResult",
        "ToolChecker",
    ],
    "dekk.detection.conda": [
        "COMMON_INSTALL_PATHS",
        "CondaDetector",
        "CondaEnvironment",
        "CondaValidation",
    ],
    # -- Configuration (core) --
    "dekk.core.config": ["ConfigManager", "ConfigReconciler", "ConfigSource"],
    # -- CI --
    "dekk.detection.ci": ["CIDetector", "CIInfo", "CIProvider", "CIBuildAdvisor", "CIBuildHints"],
    # -- Workspace --
    "dekk.detection.workspace": [
        "WorkspaceDetector",
        "WorkspaceInfo",
        "WorkspaceKind",
        "SubProject",
    ],
    # -- Versioning --
    "dekk.core.version": [
        "Version",
        "VersionSpec",
        "VersionConstraint",
        "compare_versions",
        "version_satisfies",
    ],
    "dekk.detection.version_managers": [
        "VersionManagerDetector",
        "VersionManagerInfo",
        "ManagedVersion",
    ],
    # -- Lockfiles --
    "dekk.detection.lockfile": [
        "LockfileParser",
        "LockfileInfo",
        "LockfileKind",
        "LockedDependency",
    ],
    # -- Compiler & Build --
    "dekk.detection.compiler": [
        "CompilerDetector",
        "CompilerFamily",
        "CompilerInfo",
        "ToolchainInfo",
    ],
    "dekk.detection.build": [
        "BuildSystemDetector",
        "BuildSystemInfo",
        "BuildSystem",
        "BuildTarget",
    ],
    "dekk.detection.cache": ["BuildCacheDetector", "BuildCacheInfo", "CacheKind"],
    # -- Shell --
    "dekk.shell": [
        "ShellDetector",
        "ShellInfo",
        "ShellKind",
        "ActivationScriptBuilder",
        "ActivationConfig",
        "EnvVar",
        "CompletionGenerator",
        "CompletionSpec",
        "PromptHelper",
        "AliasSuggestor",
    ],
    # -- Toolchain --
    "dekk.execution.toolchain": [
        "ToolchainProfile",
        "EnvVarBuilder",
        "CMakeToolchain",
        "CondaToolchain",
    ],
    # -- OS Abstraction --
    "dekk.execution.os": ["DekkOS", "PosixDekkOS", "WindowsDekkOS", "get_dekk_os"],
    # -- Environment --
    "dekk.execution.env": ["EnvSnapshot"],
    # -- Diagnostics --
    "dekk.diagnostics.diagnostic": [
        "DiagnosticReport",
        "DiagnosticCheck",
        "CheckRegistry",
        "DiagnosticRunner",
    ],
    "dekk.diagnostics.formatters": [
        "TextFormatter",
        "JsonFormatter",
        "MarkdownFormatter",
    ],
    "dekk.diagnostics.diagnostic_checks": [
        "PlatformCheck",
        "DependencyCheck",
        "CIEnvironmentCheck",
    ],
    # -- Library Paths --
    "dekk.detection.libpath": ["LibraryPathInfo", "LibraryPathResolver"],
    # -- Commands --
    "dekk.core.commands": [
        "CommandStatus",
        "CommandMeta",
        "CommandProvider",
        "CommandRegistry",
        "command",
    ],
    # -- Validation --
    "dekk.diagnostics.validate": [
        "CheckStatus",
        "CheckResult",
        "ValidationReport",
        "EnvironmentValidator",
    ],
    # -- Remediation --
    "dekk.diagnostics.remediate": [
        "IssueSeverity",
        "FixStatus",
        "DetectedIssue",
        "FixResult",
        "Remediator",
        "RemediatorRegistry",
    ],
    # -- Scaffold --
    "dekk.detection.scaffold": [
        "ProjectLanguage",
        "ProjectFramework",
        "ProjectType",
        "ProjectTypeDetector",
        "FileTemplate",
        "TemplateSet",
        "TemplateRegistry",
        "SetupStep",
        "SetupScript",
        "SetupScriptBuilder",
    ],
    # -- Execution Context --
    "dekk.core.context": [
        "ExecutionContext",
        "ContextWorkspaceInfo",
        "GitInfo",
        "CPUInfo",
        "GPUInfo",
        "MemoryInfo",
        "SystemLibrary",
        "ContextDiff",
    ],
    # -- CLI Framework --
    "dekk.cli.typer_app": ["Typer", "Option", "Argument", "Exit", "Context"],
    # -- CLI Commands --
    "dekk.cli.cli_commands": ["run_doctor", "run_version", "run_env"],
    # -- CLI Styling & Output --
    "dekk.cli.styles": [
        "console",
        "err_console",
        "Colors",
        "Symbols",
        "print_success",
        "print_error",
        "print_warning",
        "print_info",
        "print_debug",
        "print_header",
        "print_step",
        "print_section",
        "print_blank",
        "print_table",
        "print_numbered_list",
        "print_next_steps",
        "PROMPT_TOKENS",
    ],
    # -- CLI Output Formatting --
    "dekk.cli.output": ["OutputFormatter", "OutputFormat", "print_dep_results"],
    # -- CLI Error Handling --
    "dekk.cli.errors": [
        "DekkError",
        "ExitCodes",
        "NotFoundError",
        "ValidationError",
        "ConfigError",
        "DependencyError",
        "DekkTimeoutError",
        "DekkPermissionError",
        "DekkRuntimeError",
    ],
    # -- CLI Progress --
    "dekk.cli.progress": ["progress_bar", "spinner"],
    # -- CLI Runner --
    "dekk.cli.runner": ["RunResult", "run_logged"],
    # -- Project & Worktree --
    "dekk.project": [
        "WorktreeCreateResult",
        "WorktreeInfo",
        "create_worktree",
        "find_git_root",
        "list_worktrees",
        "prune_worktrees",
        "remove_worktree",
        "run_project_command",
    ],
    # -- Script Runner --
    "dekk.execution.runner": ["run_script"],
}

# ---------------------------------------------------------------------------
# Build reverse lookup: name -> module_path
# ---------------------------------------------------------------------------

_ATTR_TO_MODULE: dict[str, str] = {
    name: mod for mod, names in _MODULE_ATTRS.items() for name in names
}

# ---------------------------------------------------------------------------
# PEP 562 lazy loading
# ---------------------------------------------------------------------------


def __getattr__(name: str) -> Any:  # noqa: N807
    if name in _ATTR_TO_MODULE:
        import importlib

        mod_path = _ATTR_TO_MODULE[name]
        module = importlib.import_module(mod_path)
        # Bulk-cache ALL names from this module to avoid repeated __getattr__ calls
        for attr_name in _MODULE_ATTRS[mod_path]:
            try:
                globals()[attr_name] = getattr(module, attr_name)
            except AttributeError:
                pass
        return globals()[name]

    raise AttributeError(f"module 'dekk' has no attribute {name!r}")


def __dir__() -> list[str]:  # noqa: N807
    return list(_ATTR_TO_MODULE.keys()) + ["__version__"]


__all__ = sorted(set(list(_ATTR_TO_MODULE) + ["__version__"]))



"""Tests for dekk.cli.runner -- run_logged and RunResult."""

from __future__ import annotations

from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from dekk.cli.runner import _BUILD_OUTPUT_LABEL, RunResult, run_logged

# ---------------------------------------------------------------------------
# RunResult
# ---------------------------------------------------------------------------


class TestRunResult:
    def test_ok_true_when_returncode_zero(self):
        r = RunResult(returncode=0, log_path=Path("/tmp/out.log"))
        assert r.ok is True

    def test_ok_false_when_returncode_nonzero(self):
        r = RunResult(returncode=1, log_path=Path("/tmp/out.log"))
        assert r.ok is False

    def test_ok_false_negative_returncode(self):
        r = RunResult(returncode=-1, log_path=Path("/tmp/out.log"))
        assert r.ok is False

    def test_frozen(self):
        r = RunResult(returncode=0, log_path=Path("/tmp/out.log"))
        with pytest.raises((AttributeError, TypeError)):
            r.returncode = 1  # type: ignore[misc]

    def test_log_path_stored(self, tmp_path):
        p = tmp_path / "build.log"
        r = RunResult(returncode=0, log_path=p)
        assert r.log_path == p


# ---------------------------------------------------------------------------
# run_logged
# ---------------------------------------------------------------------------


def _make_completed_process(returncode=0, stdout="some output\n"):
    """Build a mock CompletedProcess with the given returncode and stdout."""
    mock = MagicMock()
    mock.returncode = returncode
    mock.stdout = stdout
    return mock


class TestRunLogged:
    """Tests for run_logged()."""

    def test_returns_run_result(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="ok\n")
        with patch("subprocess.run", return_value=proc):
            result = run_logged(
                ["echo", "hi"],
                log_path=log_path,
                label="Test",
                spinner_text="Testing...",
            )
        assert isinstance(result, RunResult)

    def test_ok_on_returncode_zero(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="success\n")
        with patch("subprocess.run", return_value=proc):
            result = run_logged(
                ["true"],
                log_path=log_path,
                label="OK run",
                spinner_text="Working...",
            )
        assert result.ok

    def test_not_ok_on_nonzero_returncode(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=1, stdout="error\n")
        with patch("subprocess.run", return_value=proc):
            result = run_logged(
                ["false"],
                log_path=log_path,
                label="Fail run",
                spinner_text="Failing...",
            )
        assert not result.ok
        assert result.returncode == 1

    def test_log_file_created(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="line one\n")
        with patch("subprocess.run", return_value=proc):
            run_logged(
                ["echo"],
                log_path=log_path,
                label="Create file",
                spinner_text="...",
            )
        assert log_path.exists()

    def test_log_contains_section_header(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="")
        with patch("subprocess.run", return_value=proc):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="My Section",
                spinner_text="...",
            )
        content = log_path.read_text()
        assert "My Section" in content

    def test_log_contains_stdout(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="build output here\n")
        with patch("subprocess.run", return_value=proc):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Build",
                spinner_text="Building...",
            )
        content = log_path.read_text()
        assert "build output here" in content

    def test_log_path_returned(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="")
        with patch("subprocess.run", return_value=proc):
            result = run_logged(
                ["cmd"],
                log_path=log_path,
                label="Test",
                spinner_text="...",
            )
        assert result.log_path == log_path.resolve()

    def test_failure_prints_build_output_label(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=1, stdout="error\n")
        with (
            patch("subprocess.run", return_value=proc),
            patch("dekk.cli.styles.print_info") as mock_print,
        ):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Label test",
                spinner_text="...",
            )
        calls = [str(c) for c in mock_print.call_args_list]
        assert any(_BUILD_OUTPUT_LABEL in c for c in calls)

    def test_success_does_not_print_build_output_label(self, tmp_path):
        """On success the spinner completing is the signal; no path noise."""
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="fine\n")
        with (
            patch("subprocess.run", return_value=proc),
            patch("dekk.cli.styles.print_info") as mock_print,
        ):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="OK",
                spinner_text="...",
            )
        calls = [str(c) for c in mock_print.call_args_list]
        assert len(calls) == 0

    def test_failure_prints_tail(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=1, stdout="line1\nline2\nerror line\n")
        with (
            patch("subprocess.run", return_value=proc),
            patch("dekk.cli.styles.print_info") as mock_print,
        ):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Fail",
                spinner_text="...",
                tail_lines=5,
            )
        printed = " ".join(str(c) for c in mock_print.call_args_list)
        assert "error line" in printed

    def test_success_does_not_print_tail(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="fine\n")
        with (
            patch("subprocess.run", return_value=proc),
            patch("dekk.cli.styles.print_info") as mock_print,
        ):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="OK",
                spinner_text="...",
                tail_lines=5,
            )
        assert mock_print.call_count == 0

    def test_tail_lines_zero_suppresses_excerpt(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=1, stdout="error line\n")
        with (
            patch("subprocess.run", return_value=proc),
            patch("dekk.cli.styles.print_info") as mock_print,
        ):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Fail no tail",
                spinner_text="...",
                tail_lines=0,
            )
        # Only the "Build output ->" line, no excerpt
        calls = [str(c) for c in mock_print.call_args_list]
        assert len(calls) == 1
        assert _BUILD_OUTPUT_LABEL in calls[0]

    def test_append_mode_adds_to_existing_log(self, tmp_path):
        log_path = tmp_path / "build.log"
        log_path.write_text("previous content\n")

        proc = _make_completed_process(returncode=0, stdout="new content\n")
        with patch("subprocess.run", return_value=proc):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Append",
                spinner_text="...",
                append=True,
            )
        content = log_path.read_text()
        assert "previous content" in content
        assert "new content" in content

    def test_overwrite_mode_replaces_existing_log(self, tmp_path):
        log_path = tmp_path / "build.log"
        log_path.write_text("old content\n")

        proc = _make_completed_process(returncode=0, stdout="fresh content\n")
        with patch("subprocess.run", return_value=proc):
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Overwrite",
                spinner_text="...",
                append=False,
            )
        content = log_path.read_text()
        assert "old content" not in content
        assert "fresh content" in content

    def test_env_forwarded_to_subprocess(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="")
        custom_env = {"MY_VAR": "hello"}
        with patch("subprocess.run", return_value=proc) as mock_run:
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="Env test",
                spinner_text="...",
                env=custom_env,
            )
        _, kwargs = mock_run.call_args
        assert kwargs["env"] == custom_env

    def test_cwd_forwarded_to_subprocess(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="")
        with patch("subprocess.run", return_value=proc) as mock_run:
            run_logged(
                ["cmd"],
                log_path=log_path,
                label="CWD test",
                spinner_text="...",
                cwd=tmp_path,
            )
        _, kwargs = mock_run.call_args
        assert kwargs["cwd"] == tmp_path

    def test_subprocess_called_with_correct_cmd(self, tmp_path):
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout="")
        with patch("subprocess.run", return_value=proc) as mock_run:
            run_logged(
                ["cargo", "build", "--release"],
                log_path=log_path,
                label="Cargo",
                spinner_text="Building...",
            )
        args, _ = mock_run.call_args
        assert args[0] == ["cargo", "build", "--release"]

    def test_none_stdout_handled_gracefully(self, tmp_path):
        """subprocess.run stdout=None should not crash."""
        log_path = tmp_path / "build.log"
        proc = _make_completed_process(returncode=0, stdout=None)
        with patch("subprocess.run", return_value=proc):
            result = run_logged(
                ["cmd"],
                log_path=log_path,
                label="None stdout",
                spinner_text="...",
            )
        assert result.ok
